
      subroutine ppinit(inpfil)
c
c----------------------------------------------------------------------
c
c   This subroutine reads in all run parameters and initial 
c   coordinates. It is also responsible for tabulating the potential
c   energy function, and generating the constraint matrix.
c
c----------------------------------------------------------------------
      
      implicit none

c     +---------------+
c     | Include files |
c     +---------------+

#include "ppmdparam1.h"
#include "ppmdparam2.h"
#include "ppmdparam3.h"

#include "box.h"
#include "box2.h"
c #include "chirpen.h"
#include "constnts.h"
#include "control.h"
#include "conv.h"
#include "derivs.h"
#include "expene.h"
#include "geom.h"
#include "listi.h"
c #include "listpen.h"
#include "paramsi.h"
#include "paramsr.h"
#include "parents.h"
c #include "pentrnt.h"
#include "port2.h"
#include "posit.h"
#include "potprm.h"
#include "props.h"
#include "scaling.h"
#include "scalpos.h"
#include "shape.h"
#include "sizes.h"
#include "strings.h"
#include "switches.h"
#include "tables.h"
#include "times.h"
#include "which.h"

#include "energy.h"
#include "gencoords.h"
c (for initial force calc only)

#include "bonds.h"
#include "coeff.h"
#include "initial.h"
#include "zero.h"
#include "steps.h"

      integer ichir_loc(maxnun,maxchain)
      common /chir_flip/ ichir_loc

c     +-----------------+
c     | Local Variables |
c     +-----------------+

      integer k,j,i,l,n,nci,ntype,lnblnk,m
     $      , ityp,jtyp, kj, nkindpen
      integer io, idum, pol
      integer iat, iskel,i44
	integer nx,ny,nz,nangletyp,ntypangle
	integer ndihtyp,ntypdih,ich

      logical penalty,ltemp,readpenparams 
c	logical first
c
      character*(*) inpfil
      character*40 oldfil,polyfil,tmpfil, dummy
      character*70 message
c
c      real*8 uimass(0:4*nun+4)
c      common /testtest/ i
      real*8 umassi,umassj,umassk
      real*8 tpot(ntbszp1),tpf(ntbszp1)
      real*8 thijk0a,thijk0c,abk0,skin,range
      real*8 c12,c6,rinv6,pers,dlt,dlt2,frs,dfrs,
     $       at,bt,ct,dt,del,delr,delsr,rsq,r,zeta,
     $       temp0,temp1,temp2,temp13,p1,p2
      real*8 sumx,sumy,sumz
      real*8 epsilread(6,0:1), sigma(6,0:1)
      real*8 xx,yy,zz
	real*8 factor
c

c--------------------------------------------------------------------
	
	Do i=1,6
	  sigma(i,0)=0.0d0
	  sigma(i,1)=0.0d0
	  epsilread(i,0)=0.0d0
	  epsilread(i,1)=0.0d0
	End Do

      OPEN(unit=5,file=inpfil,status='old')
      READ(5,*)nfreq ,runid
      READ(5,*)crayid
      tmpfil=crayid(1:lnblnk(crayid))//'out'

      OPEN(unit=10,file=tmpfil,status='unknown',form='formatted')

      message='run_date:'
      io = 10
C      CALL wtime(message,io)
      write(io,*) 'message from wtime should be written at this place'

c
      WRITE(10,50) inpfil(1:lnblnk(inpfil))
      WRITE(10,60) runid(1:lnblnk(runid)),crayid(1:lnblnk(crayid))
 40   format(/,2x,'Machine run on = ',a)
 50   format(2x,'Input data file = ',a)
 60   format(2x,'Runid = ',a,/,2x,'Crayid = ',a)

c
c     | natoms = # of atoms
c     | walmas = piston mass for volume changes (gr/mol/A**4)
c     | move = F uses constant volume
c     | uniform = T uses constant shape
c     | fixangle = T changes cell volume with fixed angles
c     |          (if variable shape is desired, 
c     |           set uniform and fixangle to F)
c     | tails = T if you are incorporating long-range corrections
c     | pretab = T if you use pretabulated values of potential
c     | temper = initial simulation temperature (K)
c     | pext = externally imposed pressure for cons.P simulation (Pa)
c     | anistr = external stress field 
c
      READ (5,*) walmas,move,uniform,fixangle,tails,pretab
      READ (5,*) temper, pext
	  do j=1,3
	    READ(5,*) (anistr(i,j),i=1,3)
	  enddo
c                                                                    
c     | ttot = total run time in ps
c     | delinit = initial time step (ps)
c     | skin = Verlet skin thickness (as multiple of sigmahh - long range)
c     | range = reduced nonbonded atom-atom potential cut-off
c     | rspne = reduced start of spline
c     | ntype = number of atom types
c     | name = name of atoms
c     | pmass = mass of atoms in g/atom
c     | polar = polarizabilities in cubic Angstroms
c     | ne = number of electrons
c     | vdwr = van der Waals radii (A)
c
      READ (5,*) ttot,delinit
      READ (5,*) skin
      READ (5,*) range
      READ (5,*) rspne
      READ (5,*) ntype
      READ (5,*) (name(i),i=1,ntype)
      READ (5,*) (grname(i),i=1,ntype)
      READ (5,*) (pmass(i),i=1,ntype)
      READ (5,*) (sigma(i,0),i=1,ntype)
      READ (5,*) (sigma(i,1),i=1,ntype)
      READ (5,*) (epsilread(i,0),i=1,ntype) 
      READ (5,*) (epsilread(i,1),i=1,ntype) 

c
c     |
c     | Calculate masses of units
c     | -Si -

      umass(1)=pmass(1) 

c     | -CH2-

      umass(2)=pmass(2) 

c     | -CH3-

      umass(3)=pmass(3)

c     |

c     |
c     | equilibrium bond lengths (1=CH,2=CH2,3=R)
c     |

	do j=1,3
	  READ (5,*) (rij0(i,j),i=1,3)
	enddo

c     | Spring constants for bond length (kcal/mol A^2)

	do j=1,3
	  READ(5,*) (kij(i,j),i=1,3)
	enddo

c     | Equilibrium bond angles (rad) not supplements
c     |	POLYDIMETHYLSILAETHYLENE:
c     |
        READ (5,*) nangletyp
      do i = 1, nangletyp
          READ (5,*) n,j,k,l
          angletype(j,k,l) = n
      enddo
        ntypangle = n
        READ (5,*) (anglename(i), i=1,ntypangle)
        READ (5,*) (thijk0(i), i=1,ntypangle)
        READ (5,*) (cijk(i),i=1,ntypangle)
      
	pi=4.0d0*atan(1.0d0)
      twopi=2.0d0*pi 
      fourpi=4.0d0*pi
	
	do i=1,ntypangle

c     | Metatropi apo moires se rad

	   thijk0(i)=pi*thijk0(i)/180.0d0
	   thijk0(i)=pi-thijk0(i)
	enddo

c     | Angle constant (kcal/mol rad**2)

c
c     | Enter rotational barrier of torsion angles
c
        READ(5,*) ndihtyp
      do i = 1,ndihtyp
        READ(5,*) m,j,k
          dihtype(j,k)=m
      enddo
        ntypdih=m
        READ (5,*) (dihname(i), i=1,ntypdih)
	  READ (5,*) (rotbar(i), i=1,ntypdih)
c
c     | constraint = T means constrain bond lengths 
c
      READ (5,*) constraint
c
c     | Get tolerances for constraint penalty functions
c     | phicall is for the bond distances
c     | psicall is for the bond velocities
c
      READ (5,*) phicall,phitol
      READ (5,*) psicall,psitol

 
      READ (5,*) tpr,tprwrite,prnbin
      READ (5,*) tsav,finsav,trajdmp
      READ (5,*) tvscal
      READ (5,*) nose
      READ (5,*) spmas
      READ (5,*) tvscaloff,tvscalon
      READ (5,*) tdampend,tdampstrt
      READ (5,*) tequil
      READ (5,*) errtol                                     
      READ (5,*) polyfil
	read (5,*) oldformat, pol
                                        
      READ (5,*) restrt,avexist    
      if(restrt) then
        READ(5,*) oldfil,avefil
      else
        READ(5,*) dummy, dummy
      endif
c                    
c	Calculate some useful quantities and then keep reading
c

c     +--------------------------+
c     | Define various constants | 
c     +--------------------------+
c     |
c     | Fx1's are the 6 value, 5th order Gear predictor coefficients for 
c     |       a 1st order equation [NOT USED].
c     | Fx2's are the 6 value, 5th order Gear predictor coefficients for 
c     |       a 2nd order equation.
c     |
c     | First corrector coefficient from: 
c     |    I.  Gear's original Treatise - f02=19.0d0/120.0d0
c     |    II. Allen and Tildesley      - f02=3.0d0/16.0d0
c
c	This is 5 value
c      f02=19.0d0/120.0d0 
      if(move.OR.nose) then
c	This is for r"=f(r',r); useful in other than the NVE case
	f02=3.0d0/16.0d0
      else
c	This is for r"=f(r)
	f02=3.0d0/20.0d0
      endif
      f12=251.0d0/360.0d0
      f32=11.0d0/18.0d0
      f42=1.0d0/6.0d0
      f52=1.0d0/60.0d0
      rad2deg=180.0d0/pi
c                                                                          
c     | sigconv = sigmaij
c     | peconv = epsilonij
c     | sansq = squared reduced distance where skin ends
c     | skntst = squared reduced skin thickness
c     | rssq = squared reduced position of spline start
c
      WRITE(10,*)'Range initialization:'
      WRITE(10,*)'     range = ', range
      WRITE(10,*)'      skin = ', skin
      WRITE(10,*)'  types (ij); old, new defns of san'

      rssq=rspne**2
      rangesq = range*range
      skntst=(0.5d0*skin*sigma(1,1))**2
c      skntst=(0.4d0*skin*sigma(1,1))**2

c
c     | Calculate coefficients of spline curve
c     | rspne = reduced start of spline

      if(rspne.ne.0.0d0) then
         rinv6=1.0d0/(rspne**6)
         pers=4.0d0*rinv6*(rinv6-1.0d0)
         dlt=range-rspne
         dltinv=1.0d0/dlt
         frs=-48.0d0*rinv6*(rinv6-0.5d0)/rspne
         dfrs=rinv6*(624.0d0*rinv6-168.0d0)/(rspne*rspne)
         a0=pers
         a1=3.0d0*pers+dlt*frs
         a2=6.0d0*pers+3.0d0*dlt*frs+dlt*dlt*dfrs*0.50d0
         spline=.true.

         af0 = -3.0d0*( a0 + a1 + a2 )
         af1 =  4.0d0*a1 + 8.0d0*a2
         af2 = -5.0d0*a2
      else
         spline=.false.
      endif
c
c     | Calculate the table values for the non-bonded potential 
c     | and force interpolation routine.  
C     | This is taken from "numerical recipes."
c
      do 180 i=1,3
         do 180 j=1,3
            delrsq(j,i)=ransq(j,i,1)/dfloat(ntbsiz)
  180 continue

      if(pretab) then

	write(6,*) 'Pretabulated potential no longer in use'
	write(6,*) 'pretab flag will be reset to F'
	pretab = .FALSE.
	
      else

         do 255 i=1,3
            do 255 j=1,3
               delrsq(j,i)=1.0d0/delrsq(j,i)
  255    continue

      endif
c         |__ pretabulated energy/force table

c     +-----------------------------------------------------+
c     | Define scaling factors (real=reduced*scaled factor) |
c     +-----------------------------------------------------+
c     |
c     | (The following are the values of the molecular units used
c     |_in the course of the simulation)

c     +-----------------+
c     | Temperature (K) |
c     +-----------------+

      tmpscl = 1.0d3/1.9872d0

c           [=] (1000 cal/kcal) (kcal/mol) / (1.9872 cal/mol K) [=] K
c
c     +-------------------+
c     | Energy (kcal/mol) |
c     +-------------------+

      enescl=1.0d0                          

c     +----------------------+
c     | Position (angstroms) |
c     +----------------------+
c
      posscl=1.0d0 
c
c     +--------------+
c     | Mass (grams) |
c     +--------------+

      masscl=pmass(1)                       
c
c     +-----------------+
c     | Time  (seconds) |
c     +-----------------+

      timscl=posscl*1.0d-10*sqrt((1.0d-3*masscl)/
     x       (4.1868d3*enescl/6.022045d23)) 

c     +---------------------------+
c     | Pressure/stress (pascals) |
c     +---------------------------+
c
      prsscl=(4.1868d3*enescl/6.022045d23)/(posscl*1.0d-10)**3        
c
c     +----------------+
c     | Volume (cm**3) |
c     +----------------+

      volscl=(posscl**3)*1.0d-24           

c     +----------------------+
c     | Picosecond time (ps) |
c     +----------------------+

      pstimscl=timscl/1.0d-12              
 
c     +--------------------------------------+
c     | "Masses" used for extended ensembles |
c     +--------------------------------------+

      spscl=enescl*pstimscl**2

      walscl =  prsscl * 6.022045d-8

c     | polyfil = new coordinate file (must generate velocities)
c     | oldfil = old restart file
c
      if(restrt) then
         WRITE(10,300)oldfil(1:lnblnk(oldfil))
      else
         WRITE(10,310)polyfil(1:lnblnk(polyfil))
      endif
 300  format(2x,'Restart file = ',a,2x,'(No coordinate file)')
 310  format(2x,'Coordinate file = ',a,2x,'(No restart file)')

      WRITE(10,320) tmpscl,enescl,posscl,timscl,prsscl,spscl,walscl
 320  format(/,2x,'             Scaling factors',/,
     x       2x,'1.0 temperture unit = ',f10.5,' degrees K',/,
     x       2x,'1.0 energy unit = ',f12.6,' Kcal/mol',/,
     x       2x,'1.0 position unit = ',f10.6,' angstroms',/,
     x       2x,'1.0 time units = ',e20.13,' seconds',/,
     x       2x,'1.0 pressure unit = ',e20.13,' pascals'/,
     x       2x,'1.0 Q mass unit (Nose) = ',e20.13,' (Kcal ps ps)/mol',
     x       /,2x,'1.0 W mass unit (Andersen) = ',e20.13,' kg/mol')      
c
c     | Write all parameters so the user can make sure 
c     | everything is okay.
c
      WRITE(10,330) walmas
      WRITE(10,340) move,uniform,fixangle,tails,constraint,
     x             nose,spmas,temper,pext,anistr,ttot,delinit
  330 format(/,2x,'Piston mass (walmas) = ',f10.2,' gr/(mol*A**4) ')
  340 format(2x,'Change volume (move) = ',l4,/,2x,
     x       'Constant shape (uniform) = ',l4,/,2x,
     x       'Fixed angles (fixangle) = ',l4,/,2x,
     x       'Long-range corrections (tails) = ',l4,/,2x,
     x       'Constrain bond lengths (constraint) =',l4,/,2x,
     x       'Use Nose constant Temp (nose) = ',l4,/,2x,
     x       'Nose Temp mass (spmas) = ',f10.5,' (Kcal ps**2)/mol',/,2x,
     x       'Temperature (temper) = ',f10.5,' K',/,2x,
     x       'External pressure (pext) = ',e10.4,' Pa',//,2x,
     x       'External forces on walls (anistr) =',2x,3f10.3,/,
     x        39x,3f10.3,/,39x,3f10.3,//,2x,
     x       'Total runtime in ps (ttot) = ',f8.3,/,2x,
     x       'Timestep in ps (delinit) = ',f10.5,/)

      if(move) then
            if(nose) then
                  if(uniform) then
                        WRITE(10,400)
                     else if(fixangle) then
                        WRITE(10,410)
                     else
                        WRITE(10,420)
                  endif
               else
                  if(uniform) then
                        WRITE(10,430)
                     else if(fixangle) then
                        WRITE(10,440)
                     else
                        WRITE(10,450)
                  endif
            endif
         else
            if(nose) then
                  WRITE(10,460)
               else
                  WRITE(10,470)
            endif
      endif

      if(constraint) then
            WRITE(10,480)
         else
            WRITE(10,490)
      endif

  400 format(2x,'*** NPT ENSEMBLE w uniform shape ***')
  410 format(2x,'*** NPT ENSEMBLE w fixed cell angles ***')
  420 format(2x,'*** NPT ENSEMBLE w variable shape ***')
  430 format(2x,'*** NPH ENSEMBLE w uniform shape ***')
  440 format(2x,'*** NPH ENSEMBLE w fixed cell angles ***')
  450 format(2x,'*** NPH ENSEMBLE w variable shape ***')
  460 format(2x,'*** NVT ENSEMBLE ***')
  470 format(2x,'*** NVE ENSEMBLE ***')
  480 format(2x,'*** Constrained Bond Lengths ***')
  490 format(2x,'*** Flexible Bond Lengths ***')

      WRITE(10,500) skin,range,rspne,ntbsiz,ntype
      WRITE(10,510) (i,name(i),pmass(i),i=1,3)
      WRITE(10,520) ((sigconv(ngbin3(i,j),0),i=1,3),j=1,3)
      WRITE(10,525) ((sigconv(ngbin3(i,j),1),i=1,3),j=1,3)
      WRITE(10,530) ((peconv(ngbin3(i,j),0),i=1,3),j=1,3)
      WRITE(10,535) ((peconv(ngbin3(i,j),1),i=1,3),j=1,3)
      WRITE(10,540) rij0(1,2),kij(1,2),thijk0(1)*rad2deg,
c     >			thijk0a*rad2deg
c     >             ,thijk0c*rad2deg,rad2deg*ak0(1),abk0*rad2deg,
     x              cijk(1),cijk(2),cijk(3),rotbar
      WRITE(10,550) phicall,psicall,phitol,psitol,tpr,tprwrite,
     x              tsav,finsav,trajdmp,tvscal,tvscaloff,tvscalon
      WRITE(10,560) tdampstrt,tdampend,tequil,errtol
  500 format(/,2x,'Reduced Verlet skin thickness (skin) = ',f10.5,/,2x,
     x       'Reduced Lennard-Jones range (range) = ',f10.5,/,2x,
     x       'Reduced start of spline (rspne) = ',f10.5,/,2x,
     x       'Potential table size (ntbsiz) = ',i5,//,2x,
     x       'Number of atom types (ntype) = ',i3,/)
  510 format(2x,'Molecule Potential parameters',/,2x,29('-'),/,
     x       3(2x,'Atom ',i2,/,'      Name = ',a8,/,
     x       '     Pmass = ',d11.5,//))
c     x       '     Pmass = ',d11.5,'    Polar = ',f10.5,/,
c     x       '        Ne = ',f10.5,'      Vdwr = ',f10.5,//))
  520 format(2x,'Sigma0 (Angs) =',19x,'H',10x,'C',10x,'R',/,
     x       25x,'H',3x,3(f10.6,1x),/,25x,'C',3x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x))
  525 format(2x,'Sigma1 (Angs) =',19x,'H',10x,'C',10x,'R',/,
     x       25x,'H',3x,3(f10.6,1x),/,25x,'C',3x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x))
  530 format(/,2x,'Epsilon0 (Kcal/mol) =',13x,'H',10x,'C',10x,'R',/,
     x       25x,'H',3x,3(f10.6,1x),/,25x,'C',3x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x))
  535 format(/,2x,'Epsilon1 (Kcal/mol) =',13x,'H',10x,'C',10x,'R',/,
     x       25x,'H',3x,3(f10.6,1x),/,25x,'C',3x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x))
  540 format(/,2x,'Bond lengths (rij0) =',12x,'CH',10x,'CH2',10x,'R',/,
     x       25x,'CH',2x,3(f10.6,1x),/,25x,'CH2',1x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x),//,2x,
     x       'Bond potentials (kij) =',10x,'CH',9x,'CH2',8x,'R',/,
     x       25x,'CH',2x,3(f10.6,1x),/,25x,'CH2',1x,3(f10.6,1x),/,
     x       25x,'R',3x,3(f10.6,1x),//,2x,
     x       'RCC angles (thijk0) = ',f10.6,1x,//,
     x       'Achiral angles (thijk0a) = ',f10.6,1x,//,
     x       'Chiral angles (thijk0c)= ',f10.6,1x,//,
     x       'RCR angles (ak0(1)) = ',f10.6,1x,//,
     x       'CCR & RCC angles (abk0) = ',f10.6,1x,//,
     x       'Bond angle pot. (cijk1) = ',f10.6,1x,//,
     x       'Bond angle pot. (cijk2) = ',f10.6,1x,//,
     x       'Bond angle pot. (cijk3) = ',f10.6,1x,//,
     x       'Rotational barrier (rotbar) = ',f10.6,' (Kcal/mol)')
  550 format(/,2x,'Calling tol. of phi (phicall)   = ',e11.5,/,2x,
     x       'Calling tol. of psi (psicall)   = ',e11.5,/,2x,
     x       'Minimizing tol. of phi (phitol) = ',e11.5,/,2x,
     x       'Minimizing tol. of psi (psitol) = ',e11.5,//,2x,
     x        'Write quantities to standard output (tpr) = ',f8.3,/,
     x        2x,'Write quantities to prn and E files (tprwrite) = ',
     x        f8.3,/,
     x        2x,'Time between x0 and x1 saves (tsav) = ',f9.3,/,2x,
     x        'Save final run info (finsav) = ',l4,/,2x,
     x        'Dump trajectory info (trajdmp) = ',l4,//,2x,
     x        'Stop velocity scaling at = ',f8.3,' ps (tvscal)',/,2x,
     x        'Length of velocity scaling off (tvscaloff) = ',f10.5,/,
     x        2x,'Length of velocity scaling on (tvscalon) = ',f10.5,/)
  560 format(2x,'Start T damping at (tdampstrt) = ',f8.3,' ps',/,2x,
     x       'Stop T damping at (tdampend)   = ',f8.3,' ps',/,2x,
     x       'Time to reach equilibrium (tequil) = ',f8.3,' ps',/,2x,
     x       'Error allowed for in accelerations (errtol) = ',f10.6,/)

      if ( (errtol.GT.0.0d0).AND.(tvscal.GT.1.0d-9) ) then
        WRITE(10,*)' ppinit: WARNING!  velocity scaling chosen '
     >           , 'with variable time step integrator enabled'
        WRITE(10,*)'   tvscal = ',tvscal,' , errtol = ',errtol,' > 0'
      endif
c
c     +-----------------------------------------------------+
c     | Scale some input values to dimensionless quantities |
c     | (i.e. measured in molecular units).                 |
c     +-----------------------------------------------------+
c     |
c     | Temperature (K)

      tstar=temper/tmpscl       

c     | Pmass (input in grams)
c     | rij0 (input in angstroms)
c     | kij (input in kcal/(mol-angstrom**2))
c     | *** THIS IS DIFFERENT FROM SYLVESTER'S CODE ***
c     | cijk (input in kcal/(mol-radian**2))

      do i=1,3
         pmass(i)=pmass(i)/masscl        
         umass(i)=umass(i)/masscl        
      enddo
c
c     | Walmas input in kg/mol
c
      walmas=walmas/walscl
c
c     | Spmas input in (kcal ps**2)/mol
c
      spmas=spmas/spscl
c
c     | All times input in picoseconds. Here they are 
c     | converted to molecular units.
c
      ttot=ttot/pstimscl        
      delinit=delinit/pstimscl        
      tpr=tpr/pstimscl
      tprwrite=tprwrite/pstimscl
      tsav=tsav/pstimscl
      tvscal=tvscal/pstimscl
      tvscaloff=tvscaloff/pstimscl
      tvscalon=tvscalon/pstimscl
      tdampend=tdampend/pstimscl
      tdampstrt=tdampstrt/pstimscl
      tequil=tequil/pstimscl
c
c     | Pressure in Pa
c
      pext=pext/prsscl
c
c     | Set initial time step information
c     | delinit is read in [=] ps 
c     | delsq=delinit**2
c     | delcu=delinit**3
c     | delqu=delinit**4
c     | delqt=delinit**5
c
      delta=delinit
      delsq=delta*delta
      delcu=delsq*delta
      delqu=delsq*delsq
      delqt=delqu*delta

c
c     | Read in penetrant parameters if npen > 0
c     | Initialize penetrant / polymer LJ parameters
c
      READ (5,*) readpenparams

      if (readpenparams) then

        READ (5,*) nkindpen
        READ (5,*) (name(ntype+i),i=1,nkindpen)
        READ (5,*) (grname(ntype+i),i=1,nkindpen)
        READ (5,*) (pmass(ntype+i),i=1,nkindpen)
        READ (5,*) (sigma(ntype+i,1),i=1,nkindpen)
        READ (5,*) (epsilread(ntype+i,1),i=1,nkindpen)

	if(nkindpen.GT.1) then

c	  Read Equilibrium Bond Lengths and then, spring constants

	  do j=2,nkindpen
	    READ (5,*) (rij0(ntype+i,ntype+j),i=2,nkindpen)
	  enddo
	  do j=2,nkindpen
	    READ (5,*) (kij(ntype+i,ntype+j),i=2,nkindpen)
	  enddo

c	  Read Equilibrium Bond Angle and then, bending constant
	
	  READ (5,*) thijk0_pen
	  thijk0_pen = pi - thijk0_pen
	  READ (5,*) cijk(4)

c	  Read Torsional Potential Constants for Penetrant (TraPPE)

	  READ (5,*) rotpen1, rotpen2, rotpen3
	
	End if
	
        do i = 1, nkindpen
          umass(i+ntype)=pmass(i+ntype)/masscl        
        enddo

      WRITE(10,*)'Range initialization for penetrant:'
      WRITE(10,*)'     range = ', range
      WRITE(10,*)'      skin = ', skin
      WRITE(10,*)'  types (ij); old, new defns of san'


	n=ntype+nkindpen

	else

	n=ntype
      endif

c                                                                          
c     | sigconv = sigmaij
c     | peconv = epsilonij
c     | sansq = squared reduced distance where skin ends
c     | skntst = squared reduced skin thickness
c     | rssq = squared reduced position of spline start
c
      do j=1,n
         do i=1,n

           k=ngbin3(i,j)
           sigconv(k,0)=(sigma(i,0)+sigma(j,0))/2.0d0

	  IF((sigconv(k,0).ne.0.0d0)) THEN

           peconv(k,0)=sqrt(epsilread(i,0)*epsilread(j,0))

           fconv(k,0)=peconv(k,0)/sigconv(k,0)**2

           rnge(i,j,0)=range*sigconv(k,0)

           ransq(i,j,0)=(rnge(i,j,0))**2

           epslon(i,j,0) = peconv(k,0)

           epbysig2(i,j,0)= fconv(k,0)

           siginv(i,j,0) = 1.0d0 / sigconv(k,0)

           siginvsq(i,j,0) = siginv(i,j,0) * siginv(i,j,0)
		
	  ENDIF

         enddo
      enddo
c
      do j=1,n
         do i=1,n

           k=ngbin3(i,j)
           sigconv(k,1)=(sigma(i,1)+sigma(j,1))/2.0d0

           peconv(k,1)=sqrt(epsilread(i,1)*epsilread(j,1))

           fconv(k,1)=peconv(k,1)/sigconv(k,1)**2

           rnge(i,j,1)=range*sigconv(k,1)

           ransq(i,j,1)=(rnge(i,j,1))**2

           sansq(i,j)=(rnge(i,j,1)+skin*sigconv(k,1))**2

           epslon(i,j,1) = peconv(k,1)

           epbysig2(i,j,1)= fconv(k,1)

           siginv(i,j,1) = 1.0d0 / sigconv(k,1)

           siginvsq(i,j,1) = siginv(i,j,1) * siginv(i,j,1)
		
         enddo
       enddo
c
c	End test-brapt: Sat 11 May 2002 19:01:37
c
c
c     | Formulate long-range potential correction from spline
c
      at=a1-3.0d0*a0
      bt=a2-3.0d0*a1+3.0d0*a0
      ct=-3.0d0*a2+3.0d0*a1-a0
      dt=3.0d0*a2-a1
        if(spline) then
          do 1600 i=1,n
            do 1600 j=1,n
               k=ngbin3(i,j)
               del=(range-rspne)*sigconv(k,1)
               delr=2.0d0*del*rspne*sigconv(k,1)
               delsr=(rspne*sigconv(k,1))**2
               splconv(k) = del*peconv(k,1)*(-del**2*a2/8.0d0+
     1                    ( del**2*dt-delr*a2)/7.0d0+
     2                    ( del**2*ct+delr*dt-delsr*a2)/6.0d0+
     3                    ( del**2*bt+delr*ct+delsr*dt)/5.0d0+
     4                    ( del**2*at+delr*bt+delsr*ct)/4.0d0+
     5                    ( del**2*a0+delr*at+delsr*bt)/3.0d0+
     6                    ( delr*a0+delsr*at)/2.0d0+delsr*a0)
 1600     continue
        else
          do 1700 i=1,21
            splconv(i)=0.0d0
 1700     continue
        endif



        do i=1,maxatms
          mctsu(i) = 0
          sutmc(i) = 0
        end do
        mctsu(maxatms+1) = 0

        if(pol.eq.1) then

          i=-1
          Do while(i+2.le.maxatms)
            i=i+2
            sutmc(i)=i+1
            mctsu(i)=0
          End Do

          i=0
          Do while(i.lt.maxatms)
            i=i+2
            sutmc(i)=i
            mctsu(i)=i
          End Do

        else if(pol.eq.2) then

          i=-1
          Do while(i+2.le.maxatms)
            i=i+2
            sutmc(i)=2*i
          End Do

          i=0
          Do while(i.lt.maxatms)
            i=i+2
            sutmc(i)=2*i-2
          End Do

c          do i=1,natmc
          do i=1,maxatmc
            mctsu(i)=0
          enddo

          i=2
          j=0
          Do while(i+4.le.maxatms)
            j=j+2
            mctsu(i)=j
            i=i+4
          End Do

        else if(pol.eq.3) then

          Do ich=1,nchains
	      k = max0(natmc(ich),natsu(ich))
		  Do i=1,k
              read(5,*) j,mctsu(j),sutmc(j)
            End Do
		End Do

        Endif

      CLOSE(unit=5)

	open(unit=97, file=crayid(1:lnblnk(crayid))//'.trj')
c
c     +-----------------------------------------+
c     | Set up initial positions and velocities |
c     +-----------------------------------------+
c     | 
c     | If restrt = .true., then read in positions, 
c     | all derivatives, and other needed arrays and 
c     | information from file = oldfil => we are
c     |_restarting from a previous run.
c
c      first=.TRUE.

      if(restrt) then
            CALL readold(oldfil)
      else
c
c        | If restrt = .false. then read in atom initial positions 
c        | from a start-up file.  Velocities are assigned randomly
c        | from an appropriate Maxwell distribution.
c        | x1,y1,z1 = v*delta*(box edgelength/sigmaHH) in units of
c        |            boxlength where v and delta are measured
c        |            in molecular coordinates
c
         CALL readnew(polyfil)
      endif
c

#ifndef NONUNIFORM
      if(h(1,2).ne.0.0d0.or.h(1,3).ne.0.0d0.or.h(2,3).ne.0.0d0) then
        write(6,*)'ppinit: nonuniform CPP flag was not set'
        write(6,*)'        That flag must be set to enable use of a'
        write(6,*)'        nonorthogonal simulation cell in formh.'
	  write(6,*)' Mallov 3exases kati sto input arxeio!!!'
        stop 'cpp flag nonuniform problem; see ppinit.F and formh.F'
      endif
#endif

c     |
c     | Use two calls to formh to prevent asking if ichir should be
c     | recalculated each time formh is called
c     |
c     | 1) calculate everything for primary atoms,
c     |   place hydrogens incorrectly since chirality isn't known yet
c     | 2) calc chirality
c     | 3) grow hydrogens correctly
c 
      CALL formh     
c
c
c     +-----------------------------+
c     | Calculation of chain masses |
c     +-----------------------------+
c
      do i=1,nchains
         chmass(i)=0.0d0
         do j=1,natmc(i)
          chmass(i)=chmass(i)+umass(identmc(j,i))
         enddo
         do j=1,natsu(i)
          chmass(i)=chmass(i)+umass(identsu(j,i))
         enddo
      enddo
c
c	NEW VERSION; does not need nch in the lrc formulae
c
c     ncomp: Number of component
c
	 do i=1,6 
        ncomp(i)=0 
       end do 

        do i=1,nchains 
         do j=1,natmc(i)
           k=identmc(j,i) 
             ncomp(k)=ncomp(k)+1 
         end do 
         do j=1,natsu(i)
           k=identsu(j,i) 
             ncomp(k)=ncomp(k)+1 
          end do 
         end do 


c     +------------------+
c     | Define filenames |
c     +------------------+

      outfil=crayid(1:lnblnk(crayid))//'prn'
      posfil=crayid(1:lnblnk(crayid))//'fin'
      savfil=crayid(1:lnblnk(crayid))//'s'
      trajfil=crayid
      disfil=crayid(1:lnblnk(crayid))//'d'

c
c     | x0,y0,z0*h are the real atomic coords 
c     | and they range from -L/2 to L/2
c     | char(12) = ^L
c
c      write(10,640) char(12),nun,natoms,nchains,
c     x              ((h(i,j),i=1,3),j=1,3)
	write(10,1640) char(12)
1640	format(a1)
	write(10,*) '  number of chains : ', nchains
	Do j=1,nchains
	  write(10,*) 'chain ',j,' mc : ',natmc(j),' su : ',natsu(j)
	End Do
	write(10,1641) ((h(i,j),i=1,3),j=1,3)
1641	format(//,2x,
     x       'Cell size (h) = ',4x,3f10.5,/,22x,3f10.5,/,
     x       22x,3f10.5)

      do 650 j=1,nchains
         write(10,655) j
         do 660 i=1,natmc(j)
            write(10,670) i,grname(identmc(i,j)),
     1            (x0mc(i,j)*h(1,1)+y0mc(i,j)*h(1,2)+z0mc(i,j)*h(1,3)),
     2            (x0mc(i,j)*h(2,1)+y0mc(i,j)*h(2,2)+z0mc(i,j)*h(2,3)),
     3            (x0mc(i,j)*h(3,1)+y0mc(i,j)*h(3,2)+z0mc(i,j)*h(3,3))
 660     continue
         do 661 i=1,natsu(j)
            write(10,670) i,grname(identsu(i,j)),
     1            (x0su(i,j)*h(1,1)+y0su(i,j)*h(1,2)+z0su(i,j)*h(1,3)),
     2            (x0su(i,j)*h(2,1)+y0su(i,j)*h(2,2)+z0su(i,j)*h(2,3)),
     3            (x0su(i,j)*h(3,1)+y0su(i,j)*h(3,2)+z0su(i,j)*h(3,3))
 661     continue
         write(10,675)
         do 680 i=1,natmc(j)
            write(10,690) i,grname(identmc(i,j)), 
     x                  (x1mc(i,j)*h(1,1)+y1mc(i,j)*h(1,2)+
     x                    z1mc(i,j)*h(1,3))
     x                  /(delta*pstimscl),
     x                  (x1mc(i,j)*h(2,1)+y1mc(i,j)*h(2,2)+
     x                    z1mc(i,j)*h(2,3))
     x                  /(delta*pstimscl),
     x                  (x1mc(i,j)*h(3,1)+y1mc(i,j)*h(3,2)+
     x                    z1mc(i,j)*h(3,3))
     x                  /(delta*pstimscl)
 680     continue
         do 681 i=1,natsu(j)
            write(10,690) i,grname(identsu(i,j)), 
     x                  (x1su(i,j)*h(1,1)+y1su(i,j)*h(1,2)+
     x                    z1su(i,j)*h(1,3))
     x                  /(delta*pstimscl),
     x                  (x1su(i,j)*h(2,1)+y1su(i,j)*h(2,2)+
     x                    z1su(i,j)*h(2,3))
     x                  /(delta*pstimscl),
     x                  (x1su(i,j)*h(3,1)+y1su(i,j)*h(3,2)+
     x                    z1su(i,j)*h(3,3))
     x                  /(delta*pstimscl)
 681     continue
 650  continue
 640  format(a1,/,2x,'# of units (nun) = ',i6,/,2x,
     x       'Number of groups (nprim)      = ',i6,/,2x,
     x       'Number of chains (nchains)     = ',i6,//,2x,
     x       'Cell size (h) = ',4x,3f10.5,/,22x,3f10.5,/,
     x       22x,3f10.5)
 655  format(/,1x,'Chain number :',i4,/,1x, 'Group',6x,'Type',11x,'x(A)'
     x       ,10x,'y(A)',10x,'z(A)')
 670  format(' ',i3,6x,a8,3x,3(f11.7,3x))
 675  format(/,' Group      Type         vx(A/ps)      vy(A/ps)',
     x       '      vz(A/ps)')
 690  format(1x,i3,6x,a8,3x,3(f11.5,3x))
c
c     | Check for any run parameters that 
c     | cause simulation conflicts.
c
      if(uniform.and.fixangle) then
         write(10,590)
         stop 'ppinit: uniform and fixangle cannot be both true'
      endif
      if(uniform.or.fixangle) then
         if(h(1,2).ne.0.0d0.or.h(1,3).ne.0.0d0.or.h(2,3).ne.0.0d0) then
            write(10,595)
            stop 'uniform or fixangle incompatible with
     &            nonorthogonal cell'
         endif
      endif
  590 format(//,2x,'*** UNIFORM and FIXANGLE can not BOTH be set to',
     x       ' TRUE ****')
  595 format(//,2x,'*** Using UNIFORM or FIXANGLE with nonorthogonal',
     x       ' cell not implemented ***')
c
c     | Calculate a few needed items like the metric tensor, 
c     | the transposed box matrix, the total mass of the box,
c     | info for the uniform dilation case, r-space positions of
c     | the atoms and box, the bond length id lists and the initial
c     | neighbor list.
c     |
c     | hinit = cell dimensions
c     | hi = inverse of h
c     | vinit = determinant of shape matrix
c
      CALL matinv(hinit,hi,vinit)
      CALL trnspse(h,ht)
      CALL mxm(ht,3,h,3,g,3)
      g11=g(1,1)
      g22=g(2,2)
      g33=g(3,3)
c
c     | This stuff is used in verlet.F and fixbond.F
c
      g12t=2.0d0*g(1,2)
      g13t=2.0d0*g(1,3)
      g23t=2.0d0*g(2,3)

      totmass=0.0d0
      do 720 j=1,nchains
            totmass=totmass+chmass(j)
 720  continue
c      do j = 1, npen
c        do i = 1, natompen
c          totmass = totmass + umass(idpen(i,j))
c        end do
c      end do


c     | Bond length minimum image (mibl)
c	Basically used in fixbond and penalty functions
c	TEMPORARARILY OUT OF USE
c
#ifdef NOTHING
	
      do 91 nci=1,nchains
         do 61 k=1,natmc(nci)+natsu(nci)-1
             i=listbli(k)
             j=listblj(k)
             xx=x0(i,nci)-x0(j,nci)
             yy=y0(i,nci)-y0(j,nci)
             zz=z0(i,nci)-z0(j,nci)
             nx=xx+xx
             ny=yy+yy
             nz=zz+zz
             mibl(k,nci)=9*nx+3*ny+nz+14
         if ((mibl(k,nci).lt.1).or.(mibl(k,nci).gt.28)) then
           write(6,*) 'ppinit:', k,nci,mibl(k,nci)
c                STOP
        endif

 61      continue
 91   continue

c
c     | Bond length minimum image for ALKANES (mibl_pen)
c       added by brapt - 260902
c
        Do nci=1,npen
          Do k=1,natompen-1
            i=k
            j=k+1
            xx=xpen0(i,nci)-xpen0(j,nci)
            yy=ypen0(i,nci)-ypen0(j,nci)
            zz=zpen0(i,nci)-zpen0(j,nci)
            nx=xx+xx
            ny=yy+yy
            nz=zz+zz
            mibl_pen(k,nci)=9*nx+3*ny+nz+14
          End Do
        End Do

#endif

c
c -----------------------------------------------------/

c     +-----------------------------------------------------+
c     | Set up initial neighbor list and minimum image info |
c     +-----------------------------------------------------+
c
C

      CALL verlet

c
c     | If this is a constrained simulation, set up the L matrix 
c     | needed to solve for the lagrange multiplier functions.  
c     | Also minimize bond lengths to correct values and bond 
c     | velocities to zero. 
c     | Realize, we are ONLY constraining bond lengths.
c     | 
C	TEMPORARARILY OUT OF USE
c
c     | The following band matrix: (comments by Sylvester and KFM)
c     |
c     | 11  12  13  14   0   0   0
c     | 21  22  23  24  25   0   0
c     | 31  32  33  34  35  36   0
c     | 41  42  43  44  45  46  47
c     |  0  52  53  54  55  56  57 
c     |  0   0  63  64  65  66  67
c     |  0   0   0  74  75  76  77 
c     | 
c     | Is stored in lmat as:
c     | 
c     |  0   0   0   0   0   0   0  
c     |  0   0   0   0   0   0   0   | Needed by the solver
c     |  0   0   0   0   0   0   0  /
c     |  0   0   0  14  25  36  47  \     
c     |  0   0  13  24  35  46  57   | Upper diagonal   
c     |  0  12  23  34  45  56  67  /     
c     | 11  22  33  44  55  66  77  -  Main diagonal      
c     | 21  32  43  54  65  76   0  \     
c     | 31  42  53  64  75   0   0   | Lower diagonal   
c     | 41  52  63  74   0   0   0  /     
c     |
c     | The following band matrix: (comments by Stelios and Christina)
c     |
c     | 11  12  13   0   0   0   0   0   0
c     | 21  22  23   0   0   0   0   0   0
c     | 31  32  33  34   0   0   0   0   0
c     |  0   0  43  44  45  46   0   0   0 
c     |  0   0   0  54  55  56   0   0   0 
c     |  0   0   0  64  65  66  67   0   0
c     |  0   0   0   0   0  76  77  78  79
c     |  0   0   0   0   0   0  87  88  89
c     |  0   0   0   0   0   0  97  98  99
c     | 
c     |
c     | Is stored in lmat as:
c     |
c     |  0   0   0   0   0   0   0   0   0 \ Needed by the solver
c     |  0   0   0   0   0   0   0   0   0 /  
c     |  0   0  13   0   0  46   0   0  79 \Upper diagonal   
c     |  0  12  23  34  45  56  67  78  89 /     
c     | 11  22  33  44  55  66  77  88  99 -  Main diagonal      
c     | 21  32  43  54  65  76  87   0   0 \     
c     | 31   0   0  64   0   0  97   0   0 / Lower diagonal   
c     |
c     |	POLYDIMETHYLSILAETHYLENE
c     |
c     OK, here are my comments:
c     |
c     | Let r14=R1, r24=R2, r34=R3, r45=R4, r58=R5, etc...
c     | then, setting masses equal to unity to keep things simple
c     | we may write:
c     |
c     | r1" ~ F1-l1*R1
c     | r2" ~ F2-l2*R2
c     | r3" ~ F3-l3*R3
c     | r4" ~ F4+l1*R1+l2*R2+l3*R3-l4*R4
c     | r5" ~ F5+l4*R4-l5*R5
c     | r6" ~ F6+l6*R6
c     | etc,...
c     |
c     | then,
c     |
c     | R1" ~ F4-F1+2*l1*R1+l2*R2+l3*R3-l4*R4
c     | R2" ~ F4-F2+l1*R1+2*L2*R2+l3*R3-l4*R4
c     | R3" ~ F4-F3+l1*R1+l2*R2+2*l3*R3-l4*R4
c     | R4" ~ F5-F4-l1*R1-l2*R2-l3*R3+2*l4*R4
c     | R5" ~ F8-F5+2*l5*R5+l6*R6+l7*R7-l8*R8
c     | etc,...
c     |
c     | so, I guess, L matrix now reads
c     |
c     |  11  12  13  14   0   0   0   0   0
c     |  21  22  23  24   0   0   0   0   0
c     |  31  32  33  34   0   0   0   0   0
c     |  41  42  43  44  45   0   0   0   0
c     |   0   0   0  54  55  56  57  58   0
c     |   0   0   0   0  65  66  67  68   0
c     |   0   0   0   0  75  76  77  78   0
c     |   0   0   0   0  85  86  87  88  89
c     | 
c     | According to the previous comments, its storage
c     | should obey the following pattern
c     |
c     |   0   0   0   0   0   0   0   0   0
c     |   0   0   0   0   0   0   0   0   0
c     |   0   0   0  14   0   0   0  58   0
c     |   0   0  13  24   0   0  57  68   0
c     |   0  12  23  34  45  56  67  78  89
c     |  11  22  33  44  55  66  77  88  99
c     |  21  32  43  54  65  76  87   0  xx
c     |  31  42   0   0  75  86   0   0  xx 
c     |  41   0   0   0  85   0   0   0  xx
c     |
c     | As regards the exact values we note that
c     |
c     | eg R1"=F4/m4-F1/m1+l1*R1(1/m4+1/m1)
c     |       +l2*R2/m4+l3*R3/m4-l4*R4/m4
c     | so
c     |
c     |   (11)=1/m4+1/m1 and (12)=(13)=-(14)=1/m4
c     |
c     (ki o 0eos bon0os...)
c

#ifdef NOTHING
      if(constraint) then

c	POLYDIMETHYLSILAETHYLENE

	 umassi=1.0d0/umass(1)
	 umassj=1.0d0/umass(2)
	 umassk=1.0d0/umass(3)

c	lines above diagonal = 3
c	lines below diagonal = 3
c	leading dimension of matrix = 10
c	these are input parameters for sgbfa and sgbsl 
c	see fixbond.F

         do nci=1,nchains

     	   do j=1,4*nun(nci) ! edw to allazw movo gia va mn xtupaei 
	     do i=1,10		 ! otav kavw compilation
		lmat(i,j,nci)=0.0d0
	     enddo
	   enddo

	  if(nun(nci).eq.1) then

	    lmat(4,4,nci)=-umassi
	    lmat(5,4,nci)=-umassi
	    lmat(6,4,nci)=-umassi
	    lmat(8,3,nci)=-umassi
	    lmat(9,2,nci)=-umassi
	    lmat(10,1,nci)=-umassi
	    lmat(5,3,nci)= umassi
	    lmat(6,3,nci)= umassi
	    lmat(6,4,nci)= umassi
	    lmat(8,1,nci)= umassi
	    lmat(8,2,nci)= umassi
	    lmat(9,1,nci)= umassi
	    lmat(7,1,nci)= umassi+umassk
	    lmat(7,2,nci)= umassi+umassk
	    lmat(7,3,nci)= umassi+umassk
	    lmat(7,4,nci)= umassi+umassk

	  else

	   do i=1,nun(nci)

	     if(i.ne.1) lmat(6,4*i-3,nci)=-umassj
	     if(i.ne.1) lmat(7,4*i-3,nci)=umassi+umassj 
	     lmat(8,4*i-3,nci)=umassi
	     lmat(9,4*i-3,nci)=umassi
	     lmat(10,4*i-3,nci)=-umassi
	     lmat(6,4*i-2,nci)=umassi
	     lmat(7,4*i-2,nci)=umassi+umassk 
	     lmat(8,4*i-2,nci)=umassi
	     lmat(9,4*i-2,nci)=-umassi
	     lmat(5,4*i-1,nci)=umassi
	     lmat(6,4*i-1,nci)=umassi
	     lmat(7,4*i-1,nci)=umassi+umassk 
	     lmat(8,4*i-1,nci)=-umassi
	     lmat(4,4*i,nci)=-umassi
	     lmat(5,4*i,nci)=-umassi
	     lmat(6,4*i,nci)=-umassi
	     if(i.ne.nun(nci)) lmat(7,4*i,nci)=umassi+umassj 
	     if(i.ne.nun(nci)) lmat(8,4*i,nci)=-umassj 

	   enddo

	   lmat(7,1,nci)=umassi+umassk
	   lmat(7,4*nun(nci),nci)=umassi+umassk

	  endif

	 enddo

c	write(6,*) 'constraint factor?'
c	READ (5,*) factor
C 	DEBUG ON AEGEAN: 151200-2
	 do nci=1,nchains
     	    do j=1,4*nun(nci)
	       do i=1,10
		  lmat(i,j,nci)= - lmat(i,j,nci) 
c	The above line is essential
	     enddo
	   enddo
         enddo
c
c	Constraint scheme for N-ALKANES (penetrants)
c
	Do nci=1,npenmx
	  Do j=1,natpmx
	    Do i=1,4
	      lmat_pen(i,j,nci)=0.0d0
	    End Do
	  End Do
	End Do
	
	if(natompen.eq.2) then
	  umassi=1.0d0/umass(6)
	else
	  umassi=1.0d0/umass(5)
	endif
	
	Do j=1,npen
	  Do i=2,natompen-2
	    lmat_pen(3,i,j)= umassi+umassi
	    lmat_pen(2,i,j)=-umassi
	    lmat_pen(4,i,j)=-umassi
	  End Do
	  lmat_pen(3,1  ,j)= umassi+1.0d0/umass(6)
	  lmat_pen(4,1  ,j)=-umassi
	  if(natompen.gt.1) lmat_pen(2,natompen-1,j)=-umassi
	  if(natompen.gt.1) lmat_pen(3,natompen-1,j)= umassi+1.0d0/umass(6)
	End Do
c
c	Because, I think Lmat used in fixbond is minus the one in EEM*s paper
c
	Do nci=1,npenmx
	  Do j=1,natpmx
	    Do i=1,4
	      lmat_pen(i,j,nci)=-lmat_pen(i,j,nci)
	    End Do
	  End Do
	End Do
c
c	The following is useful in forming amat_pen (see fixbond.F)
c
	Do i=1,natpmx
	  Do j=1,natpmx
	    band_pen(i,j)=0
	  End Do
	End Do
	band_pen(1,1) = 3
	Do i=2,natompen-1
	  band_pen(i-1,i  )=2
	  band_pen(i  ,i  )=3
	  band_pen(i  ,i-1)=4
	End Do
c	Do i=1,natompen-2
c	  Do j=2,natompen-1
c	    band_pen(i,j)=2
c	  End Do
c	End Do
c	Do i=2,natompen-1
c	  Do j=1,natompen-2
c	    band_pen(i,j)=4
c	  End Do
c	End Do
c
c        +----------------------------+ 	    
c        | calculate initial energies |
c        +----------------------------+ 	    
c

         epblsum  = 0.0d0
         epnbsum  = 0.0d0
         epbasum  = 0.0d0
         eptasum  = 0.0d0
         epfixsum = 0.0d0
         epchirsum = 0.0d0
         eppensum = 0.0d0
	   epblpen = 0.0d0
	   epbapen = 0.0d0
	   eptapen = 0.0d0
	   epnbpen = 0.0d0

         CALL calcfrcs
      
         WRITE(10,*)
         WRITE(10,*)'  initial structure energies:'
         WRITE(10,*)'      Ebend   = ', epbasum
         WRITE(10,*)'      Erot    = ', eptasum
         WRITE(10,*)'      Enb     = ', epnbsum
         WRITE(10,*)'      Ebond   = ', epblsum
         WRITE(10,*)'      Efixman = ', epfixsum
         WRITE(10,*)'      Echir = ', epchirsum
         WRITE(10,*)'      Epen    = ', eppensum
         WRITE(10,*)
        
c        +---------------------------------------------+
c        | Initialize the constraint penalty functions |
c        +---------------------------------------------+
c
         WRITE(10,930)
         ltemp=penalty()

      endif
#endif
  930 format(/,2x,'Checking penalty functions')

c     +---------------------------------------------------------+
c     | Reset any drift in the system that may have accumulated |
c     +---------------------------------------------------------+
c
      if(restrt) then
         sumx=0.0d0
         sumy=0.0d0
         sumz=0.0d0
         do j=1,nchains
           do i=1,natmc(j)
             sumx=sumx+x1mc(i,j)*umass(identmc(i,j))
             sumy=sumy+y1mc(i,j)*umass(identmc(i,j))
             sumz=sumz+z1mc(i,j)*umass(identmc(i,j))
           enddo 
           do i=1,natsu(j)
             sumx=sumx+x1su(i,j)*umass(identsu(i,j))
             sumy=sumy+y1su(i,j)*umass(identsu(i,j))
             sumz=sumz+z1su(i,j)*umass(identsu(i,j))
           enddo 
         enddo 
         sumx=sumx/totmass
         sumy=sumy/totmass
         sumz=sumz/totmass
         do j=1,nchains
           do i=1,natmc(j)
             x1mc(i,j)=x1mc(i,j)-sumx
             y1mc(i,j)=y1mc(i,j)-sumy
             z1mc(i,j)=z1mc(i,j)-sumz
           enddo
           do i=1,natsu(j)
             x1su(i,j)=x1su(i,j)-sumx
             y1su(i,j)=y1su(i,j)-sumy
             z1su(i,j)=z1su(i,j)-sumz
           enddo
	 enddo
      endif
c
      RETURN
      END
