      subroutine calcfrcs
c----------------------------------------------------------------
c
c  Calculate atomic forces, wall forces and potential energy
c  contributions based on current positions and velocities.
c
c----------------------------------------------------------------

      implicit none

c     +---------------+
c     | Include Files |
c     +---------------+

#include "ppmdparam1.h"
#include "ppmdparam2.h"
c #include "ppmdparam_pen.h"
c ppmdparam_pen.h is included in pentrnt.h
c  #include "pentrnt.h"
c  #include "bonds_pen.h"

#include "bonds.h"
#include "coeff.h"
c #include "geoder.h"
#include "geom.h"
#include "index.h"
#include "parents.h"
#include "scaling.h"
#include "shape.h"

#include "box.h"
#include "box2.h"
#include "constnts.h"
#include "conv.h"
#include "derivs.h"
#include "energy.h"
#include "expene.h"
#include "forces.h"
#include "initial.h"
#include "listi.h"
c #include "listpen.h"
#include "paramsi.h"
#include "paramsr.h"
#include "posit.h"
#include "potprm.h"
#include "print.h"
#include "props.h"
#include "scalpos.h"
#include "sizes.h"
#include "switches.h"
#include "tables.h"
#include "times.h"
#include "which.h"
#include "steps.h" 
C steps.h has been included for DEBUGGING

#include "mike.h"

c     +-----------------+
c     | Local Variables |
c     +-----------------+

      integer ichain,n
      integer i,ii,iip1,j,k, ntotal
      integer natomspi

      logical calcbl

      real*8 t1(maxatms),t2(maxatms),t3(maxatms)
     >      ,t4(maxatms),t5(maxatms),t6(maxatms)
      real*8 virlrc,virspc,pintspc,heatpx,heatpy,heatpz,gcl, deltainv
      real*8 umass1,umassinv,rsq,rljt,a,bb,temp1,temp2,uspl,
     x       rspnei,rspnei3,ssum,pmat(3) 
      real*8 momx(maxchain), momy(maxchain),momz(maxchain)
c      real*8 mompx(npenmx), mompy(npenmx),mompz(npenmx)
      real*8 xij1,yij1,zij1,xij2,yij2,zij2,pone,ppotj,pfj,
     x       temp15,temp18,temp19
	logical condition

c----------------------------------------------------------------------
c
c     | delta=time step in molecular units (NOT ps)
c     |
c     | This next routine must be done no matter which 
c     | ensemble you use.
c
c     +------------+
c     | Initialize |
c     +------------+

      do ichain = 1, nchains
        do i= 1, natmc(ichain)    
          fxmc(i,ichain)=0.d0
          fymc(i,ichain)=0.d0
          fzmc(i,ichain)=0.d0
        enddo
        do i= 1, natsu(ichain)      
          fxsu(i,ichain)=0.d0
          fysu(i,ichain)=0.d0
          fzsu(i,ichain)=0.d0
        enddo
      enddo

      
      do i=1,maxatms  
        t1(i)=0.0d0
        t2(i)=0.0d0
        t3(i)=0.0d0
        t4(i)=0.0d0
        t5(i)=0.0d0
        t6(i)=0.0d0
      end do

      do i = 1, nchains
         momx(i)=0.0d0
         momy(i)=0.0d0
         momz(i)=0.0d0
      end do
      
c      do i = 1, npen
c         mompx(i)=0.0d0
c         mompy(i)=0.0d0
c         mompz(i)=0.0d0
c      end do
      
      deltainv = 1.0d0 / delta

      if(uniform) then
         gcl=2.0d0*hl1/(delta*hl0)
         do 20 j=1,nchains
           do i=1,natmc(j) 
               umass1=umass(identmc(i,j))
c
c              | Calculate sum over nprim of rdoti*transpose(rdoti).
c              |     (R2SiC: natoms ^)           (^ ie momentum)   
c              | The elements of pimat are calculated explicitly rather
c              | than with do loops or matmul function calls to
c              | eliminate overhead (significant since the matrices
c              | and vectors involved are small). Intermediate results
c              | are stored in t# rather than added to pimat to permit
c              | vectorization and concurrency of the loop over i.
c              | They are summed at the end using vector operations.
c              |
c              | x1=delta*x'*sighh (scaled frame)
c              | x1=delta*x'       (scaled frame) (11/24/95,later versions)
c              | virmat = Virial matrix (real frame)
c
               xij1=x1mc(i,j) * deltainv 
               yij1=y1mc(i,j) * deltainv
               zij1=z1mc(i,j) * deltainv

c              | Convert velocities to molecular units
c              | sqrt(mass(H)/epphh)
c
               xij2=h(1,1)*xij1+h(1,2)*yij1+h(1,3)*zij1
               yij2=h(2,1)*xij1+h(2,2)*yij1+h(2,3)*zij1
               zij2=h(3,1)*xij1+h(3,2)*yij1+h(3,3)*zij1
               t1(i)=t1(i)+xij2*xij2*umass1 ! kinetic nrg tensor
               t2(i)=t2(i)+xij2*yij2*umass1
               t3(i)=t3(i)+xij2*zij2*umass1
               t4(i)=t4(i)+yij2*yij2*umass1
               t5(i)=t5(i)+yij2*zij2*umass1
               t6(i)=t6(i)+zij2*zij2*umass1
c
c              | Calculation of momentum of each chain
c
               momx(j)=momx(j)+xij2*umass1
               momy(j)=momy(j)+yij2*umass1
               momz(j)=momz(j)+zij2*umass1
c
c             | Calculate force on atom due to change of box shape,
c             | if any.
c
              fxmc(i,j)=-gcl*xij1*umass1*h(1,1) ! D(momentum)
              fymc(i,j)=-gcl*yij1*umass1*h(2,2) ! ----------- 
              fzmc(i,j)=-gcl*zij1*umass1*h(3,3) !     Dt 
	        end do

           do i=1,natsu(j)
			 k = natmc(j) + i
               umass1=umass(identsu(i,j))

c
               xij1=x1su(i,j) * deltainv 
               yij1=y1su(i,j) * deltainv
               zij1=z1su(i,j) * deltainv

c              | Convert velocities to molecular units
c              | sqrt(mass(H)/epphh)
c
               xij2=h(1,1)*xij1+h(1,2)*yij1+h(1,3)*zij1
               yij2=h(2,1)*xij1+h(2,2)*yij1+h(2,3)*zij1
               zij2=h(3,1)*xij1+h(3,2)*yij1+h(3,3)*zij1
               t1(k)=t1(k)+xij2*xij2*umass1 ! kinetic nrg tensor
               t2(k)=t2(k)+xij2*yij2*umass1
               t3(k)=t3(k)+xij2*zij2*umass1
               t4(k)=t4(k)+yij2*yij2*umass1
               t5(k)=t5(k)+yij2*zij2*umass1
               t6(k)=t6(k)+zij2*zij2*umass1
c
c              | Calculation of momentum of each chain
c
               momx(j)=momx(j)+xij2*umass1
               momy(j)=momy(j)+yij2*umass1
               momz(j)=momz(j)+zij2*umass1
c
c             | Calculate force on atom due to change of box shape,
c             | if any.
c
              fxsu(i,j)=-gcl*xij1*umass1*h(1,1) ! D(momentum)
              fysu(i,j)=-gcl*yij1*umass1*h(2,2) ! ----------- 
              fzsu(i,j)=-gcl*zij1*umass1*h(3,3) !     Dt 
	        end do


   20       continue


         else  ! ie if (.not.uniform)

            do 30 j=1,nchains
               do i=1,natmc(j) 
                  umass1=umass(identmc(i,j))
                  xij1=x1mc(i,j) * deltainv
                  yij1=y1mc(i,j) * deltainv
                  zij1=z1mc(i,j) * deltainv
c
c                 | Convert velocities to molecular units
c                 | sqrt(mass(H))
c
                  xij2=h(1,1)*xij1+h(1,2)*yij1+h(1,3)*zij1
                  yij2=h(2,1)*xij1+h(2,2)*yij1+h(2,3)*zij1
                  zij2=h(3,1)*xij1+h(3,2)*yij1+h(3,3)*zij1
                  t1(i)=t1(i)+xij2*xij2*umass1
                  t2(i)=t2(i)+xij2*yij2*umass1
                  t3(i)=t3(i)+xij2*zij2*umass1
                  t4(i)=t4(i)+yij2*yij2*umass1
                  t5(i)=t5(i)+yij2*zij2*umass1
                  t6(i)=t6(i)+zij2*zij2*umass1

c
c                 | Calculation of momentum of each chain
c
                  momx(j)=momx(j)+xij2*umass1
                  momy(j)=momy(j)+yij2*umass1
                  momz(j)=momz(j)+zij2*umass1
	       end do

               do i=1,natsu(j)
				k = natmc(j) + i
                  umass1=umass(identsu(i,j))
                  xij1=x1su(i,j) * deltainv
                  yij1=y1su(i,j) * deltainv
                  zij1=z1su(i,j) * deltainv
c
c
                  xij2=h(1,1)*xij1+h(1,2)*yij1+h(1,3)*zij1
                  yij2=h(2,1)*xij1+h(2,2)*yij1+h(2,3)*zij1
                  zij2=h(3,1)*xij1+h(3,2)*yij1+h(3,3)*zij1
                  t1(k)=t1(k)+xij2*xij2*umass1
                  t2(k)=t2(k)+xij2*yij2*umass1
                  t3(k)=t3(k)+xij2*zij2*umass1
                  t4(k)=t4(k)+yij2*yij2*umass1
                  t5(k)=t5(k)+yij2*zij2*umass1
                  t6(k)=t6(k)+zij2*zij2*umass1

                  momx(j)=momx(j)+xij2*umass1
                  momy(j)=momy(j)+yij2*umass1
                  momz(j)=momz(j)+zij2*umass1
	       end do

   30       continue
c

      endif
c
c     | Pimat is symmetric.
c     | Pimat = 2*momentum matrix = m*v*v 
c                                     (real frame 1/19/95)
c
c	ntotal=natoms
	ntotal=maxatms
      pimat(1,1)=ssum(ntotal,t1,1) 
      pimat(1,2)=ssum(ntotal,t2,1)
      pimat(1,3)=ssum(ntotal,t3,1)
      pimat(2,1)=pimat(1,2)
      pimat(2,2)=ssum(ntotal,t4,1)
      pimat(2,3)=ssum(ntotal,t5,1)
      pimat(3,1)=pimat(1,3)
      pimat(3,2)=pimat(2,3)
      pimat(3,3)=ssum(ntotal,t6,1)

c
      do j = 1, nchains

          momx(j)=momx(j)*momx(j)/chmass(j)
          momy(j)=momy(j)*momy(j)/chmass(j)
          momz(j)=momz(j)*momz(j)/chmass(j)

      enddo
c
c     | this is momentum matrix for moleculer virial
c
      
	(1)=ssum(nchains,momx,1)/vol
      pmat(2)=ssum(nchains,momy,1)/vol
      pmat(3)=ssum(nchains,momz,1)/vol

c     +-------------------------------------+
c     | Calculate nonbonded two body forces |
c     +-------------------------------------+

      CALL nonbond 

c     +---------------------------------------+
c     | Only calculate bond length forces if  |
c     | not using constrained dynamics.       |
c     +---------------------------------------+
c
      if(.not.constraint) then
         CALL bondlen
      endif

c     +----------------------------------------+
c     | 3 body potential and force calculation |
c     +----------------------------------------+

      CALL bondang

c     +-----------------------------+
c     | 4 body potential and forces |
c     +-----------------------------+

      CALL torsion

c     +-------------------------------------------+
c     | Fixman potential to sample flexible model |
c     +-------------------------------------------+

c      CALL fixman ! TEMPORARILY COMMENTED OUT

771   format(i8,3x,f9.3)
772   format(i8,2(3x,f9.3))
773   format(i8,3(3x,f9.3))
774   format(i8,4(3x,f9.3))
775   format(i8,5(3x,f9.3))
776   format(i8,10(3x,f9.3))

c     +-----------------------------------------------+
c     | Convert atom forces to accelerations (a=F/m). |
c     +-----------------------------------------------+

      do 50 j=1,nchains
         do i=1,natmc(j)
            umassinv=1.0d0/umass(identmc(i,j))
            fxmc(i,j)=fxmc(i,j)*umassinv
            fymc(i,j)=fymc(i,j)*umassinv
            fzmc(i,j)=fzmc(i,j)*umassinv
	   end do

         do i=1,natsu(j)
            umassinv=1.0d0/umass(identsu(i,j))
            fxsu(i,j)=fxsu(i,j)*umassinv
            fysu(i,j)=fysu(i,j)*umassinv
            fzsu(i,j)=fzsu(i,j)*umassinv
	   end do

   50 continue

c
c     +------------------------------------------+
c     | trans = 2 X Current KE of the particles  |
c     +------------------------------------------+
c
      trans=(pimat(1,1)+pimat(2,2)+pimat(3,3))
      urans=walmas*(hhh(1,1)+hhh(2,2)+hhh(3,3))

c
c     | Force on Nose-Hoover thermostat and thermostat forces on atoms
c     | [S. Nose, Mol.Phys, 52(2), 255, 1984; W. G. Hoover, Phys.Rev.A,
c     |  31(3), 1695, 1985]
c     |
c     | fkT is used and not (f+1)kT in real time [See for example S. Nose,
c     | Mol.Phys., 57(1), 187, 1986; J.Chem.Phys., 81(1), 511, 1984]
c     |
c     | s" = (s/Q)*[{sum} m(r')**2 - fkT] + (s')**2/s
c     | (note to use m(r')**2 and not m(sr')**2)
c     | r" = F/m - (s'r')/s  (for NVT)
c     |
c     | delsq = delta*delta
c     | trans = m(r')**2  ( units) [had been "molecular units"]
c
      if(nose) then
          spd=sp0*(trans-dfloat(ndof)*tstar)/spmas+
     x          (sp1*sp1/(delsq*sp0))
 
c
c         | Damp out temperature oscillations with a viscosity term if
c         | desired
c
          if(damping) spd=spd-sp1*etas/delta
 
c         +-----------------------------------------------+
c         | Correct equations of motion for accelerations |
c         +-----------------------------------------------+
c         |
c         | actual units of heatp are sqrt(mH)*(kcal/mol)*(1/delta)
c         | and of x1 srqt(mH)
c
          heatpx=sp1*h(1,1)/(sp0*delsq)
          heatpy=sp1*h(2,2)/(sp0*delsq)
          heatpz=sp1*h(3,3)/(sp0*delsq)

          do 80 j=1,nchains
	       do  i=1,natmc(j)
                fxmc(i,j)=fxmc(i,j)-heatpx*x1mc(i,j)
                fymc(i,j)=fymc(i,j)-heatpy*y1mc(i,j)
                fzmc(i,j)=fzmc(i,j)-heatpz*z1mc(i,j)
	      end do
	       do  i=1,natsu(j)
                fxsu(i,j)=fxsu(i,j)-heatpx*x1su(i,j)
                fysu(i,j)=fysu(i,j)-heatpy*y1su(i,j)
                fzsu(i,j)=fzsu(i,j)-heatpz*z1su(i,j)
	      end do
   80     continue

      endif

c
c     | If using constrained bond lengths calculate constraint forces.
c     | All other forces acting on atoms must be calculated first.
c     | Currently no constraint forces act on the penetrant
c
      if(constraint) then
c          calcbl=.false.
c          CALL fixbond(calcbl) 
      endif

c
c     | Calculate strain energy, and pv work done.
c     |    Virmat = atomic Virial matrix (real frame, i.e. F(r1-r2))
c     |             (this is actually - stress tensor)
c     |    Mvirm = molecular Virial matrix 
c     |    Pext = external pressure
c     |    virtra = trace of Virial in real units
c     |    mvirtra = trace of molecular Virial
c     |    pone, elasen, peevee are scaled with epphh
c
c       write(10,*)'virmat = ',virmat(1,1),virmat(2,2),virmat(3,3)
c       virtra=virmat(1,1)+virmat(2,2)+virmat(3,3)
       pone=pext*vol
       
c
c     | Calculate forces acting on walls
c     |    pint = internal pressure
c     |    anistr = external stress field
c
c
       call mxm(anistr,3,g,3,hhh,3)
       elasen=0.5d0*(hhh(1,1)+hhh(2,2)+hhh(3,3))+pone
       peevee=pone
C
c     | this is rho*K*T for atomic virial
c
c      temp18=nprim*nchains*tmpscl*trans*1.9872d-3
c     x       /(ndof*vol)
c 
C     | Addition of kinetic and potential parts of pressure to 
C     | obtain total pressure by virial theorem
c      do 100 i=1,3
c         do 100 j=1,3
c            temp19=pimat(i,j)+virmat(i,j)
cc            temp19=temp18+virmat(i,j)/vol
c            virmat(i,j)=temp19
c  100 continue
c
      virlrc=0.0d0
      virspc=0.0d0

      if(tails) then
        do 120 j=1,6
          do 120 i=1,6
            k=ngbin3(i,j)
c            virlrc=virlrc+ncomp(i)/vol*ncomp(j)*nchains*nchains*
c     x             peconv(k,1)*sigconv(k,1)**3
c
c	NEW VERSION (see ppinit.F) :
c
            virlrc=virlrc+ncomp(i)/vol*ncomp(j)*
     x             peconv(k,1)*sigconv(k,1)**3

	    if(pretab) then
c
c           calculation from pretabulated values
c

              rsq=(rspne*sigconv(k,1))**2*delrsq(i,j)
	      ii=int(rsq)
              if(ii.lt.ntbsiz.and.spline) then
                ii=min0(ii,ntbsiz)
                iip1=ii+1
                a=dble(iip1)-rsq
c               a=float(iip1)-rsq
                bb=1.0d0-a
                temp1=(a*a-1.0d0)
                temp2=(bb*bb*bb-bb)
                uspl=((delpot(ii)+temp1*ppot2(ii))*a+ppot(iip1)+
     x               temp2*ppot2(iip1))*peconv(k,1)
c                virspc=virspc+ncomp(i)/vol*ncomp(j)*nchains*nchains*
c     x                 ((rspne*sigconv(k,1))**3*uspl
c     x                 +3.0d0*splconv(k))
c
c	NEW VERSION (see ppinit.F) :
c
                virspc=virspc+ncomp(i)/vol*ncomp(j)*
     x                 ((rspne*sigconv(k,1))**3*uspl
     x                 +3.0d0*splconv(k))
              endif
	     
	    else
c
c             explicit calculation
c
              if(spline) then
                rsq=rspne**2
	        CALL exppot(rsq,ppotj,pfj)
	        uspl=ppotj*peconv(k,1)
c                virspc=virspc+ncomp(i)/vol*ncomp(j)*nchains*nchains*
c     x		      ((rspne*sigconv(k,1))**3*uspl
c     x                +3.0d0*splconv(k))
c
c	NEW VERSION (see ppinit.F) :
c
                virspc=virspc+ncomp(i)/vol*ncomp(j)*
     x		      ((rspne*sigconv(k,1))**3*uspl
     x                +3.0d0*splconv(k))
              endif
          
	   endif

  120    continue
         if(spline) then
               rljt=rspne
            else
               rljt=rnge(1,1,1)
         endif
         rspnei=1.0d0/rljt
         rspnei3=rspnei*rspnei*rspnei
         virlrc=virlrc*16.0d0*pi/3.0d0*rspnei3*
     x         (2.0d0*rspnei3*rspnei3/3.0d0-1.0d0)
         virspc=virspc*2.0d0*pi/3.0d0
      endif
c
c     virlrc,virspc 
c
      pintlrc=virlrc/vol
      pintspc=virspc/vol
      pintlrc=pintlrc-pintspc

c     | pint in real units
c
c      pint=(virmat(1,1)+virmat(2,2)+virmat(3,3))/3.0d0/vol
c
c      virmat is already scaled with vol
c      pint=(virmat(1,1)+virmat(2,2)+virmat(3,3))/3.0d0
c
c     +----------------------------------------------+
c     | Add contributions to molecular virial matrix |
c     +----------------------------------------------+
c
c     this is rho*K*T for molecular virial
c

c      temp15=nchains*tmpscl*trans*1.9872d-3
c     x       /(ndof*vol)


      mvirm(1,1)=mvirm(1,1)/vol+pmat(1)+pintlrc
      mvirm(1,2)=mvirm(1,2)/vol
      mvirm(1,3)=mvirm(1,3)/vol
      mvirm(2,1)=mvirm(1,2)
      mvirm(2,2)=mvirm(2,2)/vol+pmat(2)+pintlrc
      mvirm(2,3)=mvirm(2,3)/vol
      mvirm(3,1)=mvirm(1,3)
      mvirm(3,2)=mvirm(2,3)
      mvirm(3,3)=mvirm(3,3)/vol+pmat(3)+pintlrc
c	
      mpint=(mvirm(1,1)+mvirm(2,2)+mvirm(3,3))/3.0d0
c      write(10,*)'mpint',mpint

c      tramp=temp15
c      tramp1=temp18

      tramp1=(pimat(1,1)+pimat(2,2)+pimat(3,3))/3.0d0/vol
      tramp=(pmat(1)+pmat(2)+pmat(3))/3.0d0
c      write(10,*)'pmat',prsscl*tramp
c      write(*,*)'pmat',prsscl*tramp
c
      if(move) then
        if(uniform) then
c
c         | Uniform dilation - method as described in Nose and
c         | Klein's paper - modified to work within
c         | Parrinello-Rahman framework with Nose-Hoover
c         | thermostat (see Nose and Yonezawa for details)
c         | vinit = det(hinit), the shape matrix of the
c         |                     very first box
c         |
c         | l" = l**2 * det(hinit)*Tr(int.stress-Pex)/walmas
c
c          hldd=(pint+pintlrc-pext)*(3.0d0*hl0*hl0*vinit)
c     x          /(walmas*h(1,1))
c
c          hldd=(mpint-pext)*(3.0d0*hl0*hl0*vinit)
c     x          /(walmas*h(1,1))

          hldd=(mpint-pext)/(3.0d0*hl0*hl0*vinit*walmas)

c          write(*,*)'pext',1.0d0/(3.0d0*hl0*hl0*vinit*walmas) 
c
c         | Kinetic energy for wall in uniform dilation in kcal/mol
c         
          urans=walmas*hl1*hl1/delsq
        endif
      endif
c
      virtra=(virtra+trans)/3.0d0+
     x       (pintlrc*vol-peevee)
c
c     | Nose-Hoover thermostat forces on walls
c     | L"= L(NPH)*s*s + L'*s'/s
c
      if(nose) then
          if(uniform) then
                hldd = hldd*sp0*sp0 + hl1*heatpx/h(1,1)
     x               - 2.0d0*hl1*hl1/(hl0*delsq)
          endif
          urans=urans/(sp0*sp0)
      endif
c
c     | Damp out oscillations in box with a viscosity term if desired
c
      if(damping.and.move) then
          if(uniform) then
                hldd=hldd-etah*hl1/delta
          endif
      endif

      trans=0.5d0*trans
      urans=0.5d0*urans



      RETURN
      END
