
c-----subroutine vxv------------------------------------------------
c
      subroutine vxv(v1,v2,v3)
c
c  formation of the external(cross) product of two vectors, v3=v1xv2
c  subroutine is designed so that the product can be written in one of
c  the factors (v1 or v2 can be used in place of v3)
c
c      implicit double precision(a-h,o-z)
c      dimension v1(3), v2(3), v3(3)

      implicit none

      real*8 v1(3), v2(3), v3(3), vtmp(3)

      vtmp(1)=v1(2)*v2(3)-v2(2)*v1(3)
      vtmp(2)=v1(3)*v2(1)-v2(3)*v1(1)
      vtmp(3)=v1(1)*v2(2)-v2(1)*v1(2)

      v3(1)=vtmp(1)
      v3(2)=vtmp(2)
      v3(3)=vtmp(3)

      return
      end
 
 
 
c-----subroutine matmat-------------------------------------------------
c
      subroutine matmat(a,b,ab,m,n,l)
c
c  matrix multiplication a.b=ab

      implicit none

c > PASSED

      integer m,n,l
      real*8  a(m,n), b(n,l), ab(m,l)

c > LOCAL

      real*8  aux(10,10), sum
      integer i,j,k

c Multiply matrices, store in aux
c      If ( a, b, a ) was sent, we must not molest
c      matrix ab until calculations are finished

      do i = 1, m
        do j = 1, l

          sum = 0.0d+00
          do k = 1, n
            sum = sum + a(i,k)*b(k,j)
          end do
          aux(i,j) = sum

        end do
      end do

C Recover the product a.b

      do j = 1, l
        do i = 1, m
          ab(i,j) = aux(i,j)
        end do
      end do

      return
      end

c
c-----subroutine matvec-------------------------------------------------
c
      subroutine matvec(a, v, av, m, n)
c
c  matrix- vector multiplication, a.v=av
c
c      implicit double precision(a-h,o-z)
c      dimension a(m,n), v(n), av(m)

      implicit none

      integer m,n,i,k
      real*8  a(m,n), v(n), av(m)

      do 10 i=1,m
         av(i)=0.0d+00
         do 20 k=1,n
            av(i)=av(i)+a(i,k)*v(k)
   20    continue
   10 continue
      return
      end


c-----subroutine formmatA-----------------------------------------------
c
      subroutine formA(R,A)

      implicit none
      real*8 R(3),A(3,3)

      A(1,1)=0.0d0
      A(1,2)=-R(3)
      A(1,3)=R(2)
      A(2,1)=R(3)
      A(2,2)=0.0d0
      A(2,3)=-R(1)
      A(3,1)=-R(2)
      A(3,2)=R(1)
      A(3,3)=0.0d0

      return
      end


c-----subroutine dyadic-----------------------------------------------

      subroutine dyadic(v1,v2,v3)

      implicit none

      real*8 v1(3),v2(3),v3(3,3)

      v3(1,1)=v1(1)*v2(1)
      v3(1,2)=v1(1)*v2(2)
      v3(1,3)=v1(1)*v2(3)
      v3(2,1)=v1(2)*v2(1)
      v3(2,2)=v1(2)*v2(2)
      v3(2,3)=v1(2)*v2(3)
      v3(3,1)=v1(3)*v2(1)
      v3(3,2)=v1(3)*v2(2)
      v3(3,3)=v1(3)*v2(3)

      return
      end

c-----subroutine mxm-------------------------------------------------
c
      subroutine mxm(a,m,b,n,ab,l)
c
c  matrix multiplication a.b=ab
c
c      implicit double precision(a-h,o-z)
c      dimension a(m,n), b(n,l), ab(m,l)

      implicit none

      integer m,n,l,i,j,k
      real*8  a(m,n), b(n,l), ab(m,l)

      do 30 i=1,m
         do 20 j=1,l
            ab(i,j)=0.0d+00
            do 10 k=1,n
               ab(i,j)=ab(i,j)+a(i,k)*b(k,j)
   10       continue
   20    continue
   30 continue
      return
      end
