 
      subroutine nonbond

c-----------------------------------------------------------------------
c
c  Nonbonded pair potentials and forces. A finite range 6-12
c  potential is used. Atoms less than three bonds apart are
c  excluded from this calculation.
c
c  At rij**2 <= rssq = rspne**2, straight 6-12 is used.
c  At rssq < rij**2 <= rangesq = range**2 , a quintic spline is used. 
c         The spline and it's first two derivatives are continous
c         with the 6-12 at rspne and go to zero at range.
c  At rij**2 > rangesq, everything is zero.
c
c  Atomic virial calculations are commented out. Check them
c  and also write them for same chain interactions!!
c
c-----------------------------------------------------------------------

      implicit none

c     +--------------+
c     | Include File |
c     +--------------+

#include "ppmdparam1.h"
#include "ppmdparam2.h"

#include "box.h"
#include "energy.h"
#include "expene.h"
#include "forces.h"
#include "index.h"
#include "listi.h"
#include "paramsr.h"
#include "parents.h"
#include "posit.h"
#include "potprm.h"
#include "scalpos.h"
#include "sizes.h"
#include "switches.h"
#include "tables.h"
#include "which.h"
#include "steps.h"

c     +-----------------+
c     | Local Variables |
c     +-----------------+

      integer ipri,iprj
      integer nci,ncj,i,ii,jlow,jhgh,j,jj,idatom1,idatom2,
     x        k,ki,l,mi,mj,cendi,cendj,coli,colj
      integer nt1(maxatms),jjj
      integer type1, type2, this, ind

      logical inter

c
      real*8 xij,yij,zij,rij,xpij,ypij,zpij,rsq,pairf
      real*8 ssum,prodrm(3),crit
      real*8 pairp,fpx,fpy,fpz
      real*8 ppotj,pfj, h11inv, h22inv, h33inv
      real*8 rinv6,r,zeta,temp1,temp2,temp12, invrr
	real*8 intranb
      real*8 tm1,tm2,tm3,tm4,tm5,tm6,
     x       cmxj,cmyj,cmzj,cmxi,cmyi,cmzi


c-----------------------------------------------------------------------
c
c     +------------+
c     | Initialize |
c     +------------+

c     Coefficients for molecular virial
c 
       tm1=0.0d0
       tm2=0.0d0
       tm3=0.0d0
       tm4=0.0d0
       tm5=0.0d0
       tm6=0.0d0

      h11inv = 1.0d0 / h(1,1)
      h22inv = 1.0d0 / h(2,2)
      h33inv = 1.0d0 / h(3,3)

	epnbsum=0.0d0

c     +-------------------------------------+
c     | Center of mass of each parent chain |
c     | used in molecular virial            |
c     +-------------------------------------+
c
      do nci=1,nchains
        prodrm(1)=0.0d0
        prodrm(2)=0.0d0
        prodrm(3)=0.0d0
        do i=1,natmc(nci)
           prodrm(1)=prodrm(1)+rpxmc(i,nci)*umass(identmc(i,nci))
           prodrm(2)=prodrm(2)+rpymc(i,nci)*umass(identmc(i,nci))
           prodrm(3)=prodrm(3)+rpzmc(i,nci)*umass(identmc(i,nci))
        enddo

         do i=1,natsu(nci)
           prodrm(1)=prodrm(1)+rpxsu(i,nci)*umass(identsu(i,nci))
           prodrm(2)=prodrm(2)+rpysu(i,nci)*umass(identsu(i,nci))
           prodrm(3)=prodrm(3)+rpzsu(i,nci)*umass(identsu(i,nci))
        enddo
        cmx(nci)=prodrm(1)/chmass(nci)
        cmy(nci)=prodrm(2)/chmass(nci)
        cmz(nci)=prodrm(3)/chmass(nci)
c        write(10,*) 'cenmass',nci,cmx(nci),cmy(nci),cmz(nci)
      enddo

c     +-----------------------------------------------+
c     | Calculation of nonbonded potential and forces |
c     +-----------------------------------------------+
c     |
c     +-------------------------+
c     | Same Chain Interactions |
c     +-------------------------+
c
      do nci=1,nchains

        do i=1,natmc(nci)-3
c	  do i=1,1
	    jlow=markermcmc(i,nci,nci)+1
	    jhgh=markermcmc(i+1,nci,nci)
	    jj=jhgh-jlow+1
          if(jj.ge.1) then
	      idatom1=identmc(i,nci) 
c	      do j=i+3,natmc(nci) ! without Verlet list
c             idatom2=identmc(j,nci) ! without Verlet list
            do j=1,jj
	        l=listnb(j-1+jlow)
	        idatom2=identmc(l,nci)
  	        k=l-i
	        ind=0
	        if(k.gt.4) ind=1 
              xij=rxmc(i,nci)-rxmc(l,nci)
              yij=rymc(i,nci)-rymc(l,nci)
              zij=rzmc(i,nci)-rzmc(l,nci)

              xij=xij-h(1,1)*dnint(xij*h11inv)
              yij=yij-h(2,2)*dnint(yij*h22inv)
              zij=zij-h(3,3)*dnint(zij*h33inv)
              rij=xij*xij+yij*yij+zij*zij

 

c             +----------------------------------+
c             | Calculation of forces explicitly |     
c             +----------------------------------+
c
			CALL Nonbondforce(rij,idatom1,idatom2,ind,pairp,pairf)

	
c			if(nstep.eq.0) write(300,999) idatom1,idatom2,rij,pairp

			epnbsum=epnbsum+pairp

c------------------------------------------------------------
c
c             +--------------------------------------------------+
c             | Calculate x,y,z components of force on atom pair |
c             +--------------------------------------------------+

			fpx=pairf*xij
			fpy=pairf*yij
			fpz=pairf*zij

			fxmc(i,nci)=fxmc(i,nci)+fpx
			fxmc(l,nci)=fxmc(l,nci)-fpx
			fymc(i,nci)=fymc(i,nci)+fpy
			fymc(l,nci)=fymc(l,nci)-fpy
			fzmc(i,nci)=fzmc(i,nci)+fpz
			fzmc(l,nci)=fzmc(l,nci)-fpz
c
c             +-----------------------------------+
c             | Calculations for molecular virial |
c             +-----------------------------------+
c
			inter=.false.

			xpij=rpxmc(i,nci)-rpxmc(l,nci)
			ypij=rpymc(i,nci)-rpymc(l,nci)
			zpij=rpzmc(i,nci)-rpzmc(l,nci)
 
			crit=dabs(xpij-xij)+dabs(ypij-yij)+
     x			   dabs(zpij-zij)
 
			if(crit.gt.1.0d-4) then 
c               write(10,*)' nci',nci,'  ncj',ncj,' i', i,'  j',j
c               write(10,*) 'crit in 1', crit
                inter=.true.
			endif 
c
c      Calculations for molecular virial
c
		  	if(inter) then 
 
			  cmxj=cmx(nci)+rxmc(i,nci)-xij-rpxmc(l,nci)
                cmyj=cmy(nci)+rymc(i,nci)-yij-rpymc(l,nci)
                cmzj=cmz(nci)+rzmc(i,nci)-zij-rpzmc(l,nci)

                cmxi=cmx(nci)+rxmc(i,nci)-rpxmc(i,nci)
                cmyi=cmy(nci)+rymc(i,nci)-rpymc(i,nci)
                cmzi=cmz(nci)+rzmc(i,nci)-rpzmc(i,nci)

                tm1=tm1+(cmxi-cmxj)*fpx
                tm2=tm2+(cmxi-cmxj)*fpy
                tm3=tm3+(cmxi-cmxj)*fpz
                tm4=tm4+(cmyi-cmyj)*fpy
                tm5=tm5+(cmyi-cmyj)*fpz
                tm6=tm6+(cmzi-cmzj)*fpz
 
			endif 

		  end do
c              |___ inner loop over atoms on chain nci
c         
		end if

        enddo 
c           |___ outer loop over atoms on chain nci
		

999	format(5(1x,g12.5))

        do i=1,natmc(nci)
c	  do i=9,9
          jlow=markermcsu(i,nci,nci)+1
	    jhgh=markermcsu(i+1,nci,nci)
	    jj=jhgh-jlow+1
          if(jj.ge.1) then
	      idatom1=identmc(i,nci) 
c	      do j=1,natsu(nci) ! without Verlet
c             idatom2=identsu(j,nci)! Without Verlet
            do j=1,jj
			l=listnb(j-1+jlow)
			idatom2=identsu(l,nci)
			k=iabs(i-sutmc(l))
			if(k.lt.2) go to 1
			  ind=0
			  if(k.gt.3) ind=1
			  xij=rxmc(i,nci)-rxsu(l,nci)
			  yij=rymc(i,nci)-rysu(l,nci)
			  zij=rzmc(i,nci)-rzsu(l,nci)

c			| rfx,rfy,rfz are the real space positions
c             | pair-pair parameters are calc'd from
c             | peconv(n), fconv(n), sigconv(n)
c
			  xij=xij-h(1,1)*dnint(xij*h11inv)
			  yij=yij-h(2,2)*dnint(yij*h22inv)
			  zij=zij-h(3,3)*dnint(zij*h33inv)
			  rij=xij*xij+yij*yij+zij*zij

c               +----------------------------------+
c               | Calculation of forces explicitly |     
c               +----------------------------------+
c
			  CALL 
     &		  Nonbondforce(rij,idatom1,idatom2,ind,pairp,pairf)

			  epnbsum=epnbsum+pairp
	
c-----------------------------------------------------------------
c
c             +--------------------------------------------------+
c             | Calculate x,y,z components of force on atom pair |
c             +--------------------------------------------------+

			  fpx=pairf*xij
			  fpy=pairf*yij
			  fpz=pairf*zij

			  fxmc(i,nci)=fxmc(i,nci)+fpx
			  fxsu(l,nci)=fxsu(l,nci)-fpx
			  fymc(i,nci)=fymc(i,nci)+fpy
			  fysu(l,nci)=fysu(l,nci)-fpy
			  fzmc(i,nci)=fzmc(i,nci)+fpz
			  fzsu(l,nci)=fzsu(l,nci)-fpz
c
c             +-----------------------------------+
c             | Calculations for molecular virial |
c             +-----------------------------------+
c
                inter=.false.

                xpij=rpxmc(i,nci)-rpxsu(l,nci)
                ypij=rpymc(i,nci)-rpysu(l,nci)
                zpij=rpzmc(i,nci)-rpzsu(l,nci)
 
                crit=dabs(xpij-xij)+dabs(ypij-yij)+
     x               dabs(zpij-zij)
 
                if(crit.gt.1.0d-4) then 
c                 write(10,*)' nci',nci,'  ncj',ncj,' i', i,'  j',j
c                 write(10,*) 'crit in 1', crit
                  inter=.true.
                endif 

                if(inter) then 
 
                  cmxj=cmx(nci)+rxmc(i,nci)-xij-rpxsu(l,nci)
                  cmyj=cmy(nci)+rymc(i,nci)-yij-rpysu(l,nci)
                  cmzj=cmz(nci)+rzmc(i,nci)-zij-rpzsu(l,nci)

                  cmxi=cmx(nci)+rxmc(i,nci)-rpxmc(i,nci)
                  cmyi=cmy(nci)+rymc(i,nci)-rpymc(i,nci)
                  cmzi=cmz(nci)+rzmc(i,nci)-rpzmc(i,nci)

                  tm1=tm1+(cmxi-cmxj)*fpx
                  tm2=tm2+(cmxi-cmxj)*fpy
                  tm3=tm3+(cmxi-cmxj)*fpz
                  tm4=tm4+(cmyi-cmyj)*fpy
                  tm5=tm5+(cmyi-cmyj)*fpz
                  tm6=tm6+(cmzi-cmzj)*fpz
 
			  endif
			 
1	CONTINUE

            end do
c                |___ inner loop over atoms on chain 2
c         
		endif

        enddo 
c           |___ end of loop over atoms on chain 1     
             
	  do i=1,natsu(nci)-2
c	  do i=6,6
          jlow=markersusu(i,nci,nci)+1
	    jhgh=markersusu(i+1,nci,nci)
	    jj=jhgh-jlow+1
          if(jj.ge.1) then
            idatom1=identsu(i,nci) 
c	      do j=i+2,natsu(nci)! without Verlet
c             idatom2=identsu(j,nci)! without Verlet
            do j=1,jj
	        l=listnb(j-1+jlow)
	        idatom2=identsu(l,nci)
 	        k=sutmc(l)-sutmc(i)
		    ind=0
	        if(k.gt.2) ind=1 
              xij=rxsu(i,nci)-rxsu(l,nci)
              yij=rysu(i,nci)-rysu(l,nci)
              zij=rzsu(i,nci)-rzsu(l,nci)

c             | rfx,rfy,rfz are the real space positions
c             | pair-pair parameters are calc'd from
c             | peconv(n), fconv(n), sigconv(n)
c
              xij=xij-h(1,1)*dnint(xij*h11inv)
              yij=yij-h(2,2)*dnint(yij*h22inv)
              zij=zij-h(3,3)*dnint(zij*h33inv)
              rij=xij*xij+yij*yij+zij*zij

c             +----------------------------------+
c             | Calculation of forces explicitly |     
c             +----------------------------------+
c
	        CALL 
     &	    Nonbondforce(rij,idatom1,idatom2,ind,pairp,pairf)

c             if(nstep.eq.0) write(300,999) idatom1,idatom2,rij,pairp

		    epnbsum=epnbsum+pairp

c-----------------------------------------------------------------
c
c             +--------------------------------------------------+
c             | Calculate x,y,z components of force on atom pair |
c             +--------------------------------------------------+

              fpx=pairf*xij
              fpy=pairf*yij
              fpz=pairf*zij

	        fxsu(i,nci)=fxsu(i,nci)+fpx
	        fxsu(l,nci)=fxsu(l,nci)-fpx
	        fysu(i,nci)=fysu(i,nci)+fpy
	        fysu(l,nci)=fysu(l,nci)-fpy
	        fzsu(i,nci)=fzsu(i,nci)+fpz
	        fzsu(l,nci)=fzsu(l,nci)-fpz
c
c             +-----------------------------------+
c             | Calculations for molecular virial |
c             +-----------------------------------+
c
              inter=.false.

              xpij=rpxsu(i,nci)-rpxsu(l,nci)
              ypij=rpysu(i,nci)-rpysu(l,nci)
              zpij=rpzsu(i,nci)-rpzsu(l,nci)
 
              crit=dabs(xpij-xij)+dabs(ypij-yij)+
     x             dabs(zpij-zij)
 
              if(crit.gt.1.0d-4) then 
c               write(10,*)' nci',nci,'  ncj',ncj,' i', i,'  j',j
c               write(10,*) 'crit in 1', crit
                inter=.true.
              endif 

              if(inter) then 
 
                cmxj=cmx(nci)+rxsu(i,nci)-xij-rpxsu(l,nci)
                cmyj=cmy(nci)+rysu(i,nci)-yij-rpysu(l,nci)
                cmzj=cmz(nci)+rzsu(i,nci)-zij-rpzsu(l,nci)

                cmxi=cmx(nci)+rxsu(i,nci)-rpxsu(i,nci)
                cmyi=cmy(nci)+rysu(i,nci)-rpysu(i,nci)
                cmzi=cmz(nci)+rzsu(i,nci)-rpzsu(i,nci)

                tm1=tm1+(cmxi-cmxj)*fpx
                tm2=tm2+(cmxi-cmxj)*fpy
                tm3=tm3+(cmxi-cmxj)*fpz
                tm4=tm4+(cmyi-cmyj)*fpy
                tm5=tm5+(cmyi-cmyj)*fpz
                tm6=tm6+(cmzi-cmzj)*fpz
 
              endif 

            end do
c              |___ inner loop over atoms on chain 2
c         
	    endif

        enddo 
c         |___ end of loop over atoms on chain 1
      enddo
c            |___ end of loop over chain 1

c       if(nstep.eq.0) write(200,*) i, idatom1,jj, idatom2, epnbsum
	
	intranb=0.0d0
	intranb=epnbsum
c
c     +---------------------------------------+
c     | Interactions between different chains | 
c     +---------------------------------------+
c
      do nci=1,nchains-1
        do ncj=nci+1,nchains
          do i=1,natmc(nci)
	      jlow=markermcmc(i,nci,ncj)+1
	      jhgh=markermcmc(i+1,nci,ncj)
	      jj=jhgh-jlow+1
            if(jj.ge.1) then
	        idatom1=identmc(i,nci) 
c	        do j=1,natmc(ncj) ! without Verlet list
c               idatom2=identmc(j,ncj)! without Verlet
			do j=1,jj
			  l=listnb(j-1+jlow)
			  idatom2=identmc(l,ncj)

                xij=rxmc(i,nci)-rxmc(l,ncj)
                yij=rymc(i,nci)-rymc(l,ncj)
                zij=rzmc(i,nci)-rzmc(l,ncj)

c               | rfx,rfy,rfz are the real space positions
c               | pair-pair parameters are calc'd from
c               |      peconv(n), fconv(n), sigconv(n)
c
                xij=xij-h(1,1)*dnint(xij*h11inv)
                yij=yij-h(2,2)*dnint(yij*h22inv)
                zij=zij-h(3,3)*dnint(zij*h33inv)
                rij=xij*xij+yij*yij+zij*zij

c----------------------------------------------------------

c               +----------------------------------+
c               | Calculation of forces explicitly |    
c               +----------------------------------+

			  CALL 
     &		  Nonbondforce(rij,idatom1,idatom2,1,pairp,pairf)

			  epnbsum=epnbsum+pairp

c               +--------------------------------------------------+
c               | Calculate x,y,z components of force on atom pair |
c               +--------------------------------------------------+
c
                fpx=pairf*xij
                fpy=pairf*yij
                fpz=pairf*zij

		      fxmc(i,nci)=fxmc(i,nci)+fpx
			  fxmc(l,ncj)=fxmc(l,ncj)-fpx
			  fymc(i,nci)=fymc(i,nci)+fpy
			  fymc(l,ncj)=fymc(l,ncj)-fpy
		      fzmc(i,nci)=fzmc(i,nci)+fpz
		      fzmc(l,ncj)=fzmc(l,ncj)-fpz
c
c               +-----------------------------------+
c               | Calculations for molecular virial |
c               +-----------------------------------+
c
                cmxj=cmx(ncj)+rxmc(i,nci)-xij-rpxmc(l,ncj)
                cmyj=cmy(ncj)+rymc(i,nci)-yij-rpymc(l,ncj)
                cmzj=cmz(ncj)+rzmc(i,nci)-zij-rpzmc(l,ncj)

                cmxi=cmx(nci)+rxmc(i,nci)-rpxmc(i,nci)
                cmyi=cmy(nci)+rymc(i,nci)-rpymc(i,nci)
                cmzi=cmz(nci)+rzmc(i,nci)-rpzmc(i,nci)

                tm1=tm1+(cmxi-cmxj)*fpx
                tm2=tm2+(cmxi-cmxj)*fpy
                tm3=tm3+(cmxi-cmxj)*fpz
                tm4=tm4+(cmyi-cmyj)*fpy
                tm5=tm5+(cmyi-cmyj)*fpz
                tm6=tm6+(cmzi-cmzj)*fpz


	        end do
c                  |___ inner loop over atoms on chain 2
c         
		  endif

          enddo 
c             |___ end of loop over atoms on chain 1
          
c         if(nstep.eq.0) write(200,*) i, idatom1,jj, idatom2, epnbsum

	    do i=1,natsu(nci)
            jlow=markersusu(i,nci,ncj)+1
		  jhgh=markersusu(i+1,nci,ncj)
		  jj=jhgh-jlow+1
		  if(jj.ge.1) then
		    idatom1=identsu(i,nci) 
c		    do j=1,natsu(ncj) ! without Verlet
c	          idatom2=identsu(j,ncj)! without verlet
		    do j=1,jj
			  l=listnb(j-1+jlow)
			  idatom2=identsu(l,ncj)

                xij=rxsu(i,nci)-rxsu(l,ncj)
                yij=rysu(i,nci)-rysu(l,ncj)
                zij=rzsu(i,nci)-rzsu(l,ncj)

c               | rfx,rfy,rfz are the real space positions
c               | pair-pair parameters are calc'd from
c               |      peconv(n), fconv(n), sigconv(n)
c
                xij=xij-h(1,1)*dnint(xij*h11inv)
                yij=yij-h(2,2)*dnint(yij*h22inv)
                zij=zij-h(3,3)*dnint(zij*h33inv)
                rij=xij*xij+yij*yij+zij*zij

c----------------------------------------------------------

c               +----------------------------------+
c               | Calculation of forces explicitly |    
c               +----------------------------------+

			  CALL
     &		  Nonbondforce(rij,idatom1,idatom2,1,pairp,pairf)

			  epnbsum=epnbsum+pairp

c               +--------------------------------------------------+
c               | Calculate x,y,z components of force on atom pair |
c               +--------------------------------------------------+
c
                fpx=pairf*xij
                fpy=pairf*yij
                fpz=pairf*zij

		      fxsu(i,nci)=fxsu(i,nci)+fpx
		      fxsu(l,ncj)=fxsu(l,ncj)-fpx
		      fysu(i,nci)=fysu(i,nci)+fpy
		      fysu(l,ncj)=fysu(l,ncj)-fpy
		      fzsu(i,nci)=fzsu(i,nci)+fpz
		      fzsu(l,ncj)=fzsu(l,ncj)-fpz
c
c               +-----------------------------------+
c               | Calculations for molecular virial |
c               +-----------------------------------+
c
                cmxj=cmx(ncj)+rxsu(i,nci)-xij-rpxsu(l,ncj)
                cmyj=cmy(ncj)+rysu(i,nci)-yij-rpysu(l,ncj)
                cmzj=cmz(ncj)+rzsu(i,nci)-zij-rpzsu(l,ncj)

                cmxi=cmx(nci)+rxsu(i,nci)-rpxsu(i,nci)
                cmyi=cmy(nci)+rysu(i,nci)-rpysu(i,nci)
                cmzi=cmz(nci)+rzsu(i,nci)-rpzsu(i,nci)

                tm1=tm1+(cmxi-cmxj)*fpx
                tm2=tm2+(cmxi-cmxj)*fpy
                tm3=tm3+(cmxi-cmxj)*fpz
                tm4=tm4+(cmyi-cmyj)*fpy
                tm5=tm5+(cmyi-cmyj)*fpz
                tm6=tm6+(cmzi-cmzj)*fpz


		    end do
c                  |___ inner loop over atoms on chain 2
c         
		  endif
          enddo 
c             |___ end of loop over atoms on chain 1
        enddo
c           |___ end of loop over chain 2
      enddo
c         |___ end of loop over chain 1

	do nci=1,nchains
	  do ncj=1,nchains
	    if(nci.ne.ncj) then
		  do i=1,natmc(nci)
		    jlow=markermcsu(i,nci,ncj)+1
			jhgh=markermcsu(i+1,nci,ncj)
		    jj=jhgh-jlow+1
			if(jj.ge.1) then
			  idatom1=identmc(i,nci) 

c	          do j=1,natsu(ncj) ! without verlet
c                 idatom2=identsu(j,ncj)! without verlet
			  do j=1,jj
			    l=listnb(j-1+jlow)
			    idatom2=identsu(l,ncj)

                  xij=rxmc(i,nci)-rxsu(l,ncj)
                  yij=rymc(i,nci)-rysu(l,ncj)
                  zij=rzmc(i,nci)-rzsu(l,ncj)

c               | rfx,rfy,rfz are the real space positions
c               | pair-pair parameters are calc'd from
c               | peconv(n), fconv(n), sigconv(n)
c
                  xij=xij-h(1,1)*dnint(xij*h11inv)
                  yij=yij-h(2,2)*dnint(yij*h22inv)
                  zij=zij-h(3,3)*dnint(zij*h33inv)
                  rij=xij*xij+yij*yij+zij*zij

c----------------------------------------------------------

c                 +----------------------------------+
c                 | Calculation of forces explicitly |    
c                 +----------------------------------+

	            CALL
     &		    Nonbondforce(rij,idatom1,idatom2,1,pairp,pairf)

			    epnbsum=epnbsum+pairp

c                 +--------------------------------------------------+
c                 | Calculate x,y,z components of force on atom pair |
c                 +--------------------------------------------------+
c
                  fpx=pairf*xij
                  fpy=pairf*yij
                  fpz=pairf*zij

		        fxmc(i,nci)=fxmc(i,nci)+fpx
		        fxsu(l,ncj)=fxsu(l,ncj)-fpx
			    fymc(i,nci)=fymc(i,nci)+fpy
		        fysu(l,ncj)=fysu(l,ncj)-fpy
		        fzmc(i,nci)=fzmc(i,nci)+fpz
			    fzsu(l,ncj)=fzsu(l,ncj)-fpz
c
c                 +-----------------------------------+
c                 | Calculations for molecular virial |
c                 +-----------------------------------+
c
                  cmxj=cmx(ncj)+rxmc(i,nci)-xij-rpxsu(l,ncj)
                  cmyj=cmy(ncj)+rymc(i,nci)-yij-rpysu(l,ncj)
                  cmzj=cmz(ncj)+rzmc(i,nci)-zij-rpzsu(l,ncj)

                  cmxi=cmx(nci)+rxmc(i,nci)-rpxmc(i,nci)
                  cmyi=cmy(nci)+rymc(i,nci)-rpymc(i,nci)
                  cmzi=cmz(nci)+rzmc(i,nci)-rpzmc(i,nci)

                  tm1=tm1+(cmxi-cmxj)*fpx
                  tm2=tm2+(cmxi-cmxj)*fpy
                  tm3=tm3+(cmxi-cmxj)*fpz
                  tm4=tm4+(cmyi-cmyj)*fpy
                  tm5=tm5+(cmyi-cmyj)*fpz
                  tm6=tm6+(cmzi-cmzj)*fpz

		      end do
c                    |___ inner loop over atoms on chain 2
		    endif

            enddo 
c               |___ end of loop over atoms on chain 1
	    endif

	  enddo

	enddo
 
c
c      Add contributions to molecular virial matrix
c
      mvirm(1,1)=mvirm(1,1)+tm1
      mvirm(1,2)=mvirm(1,2)+tm2
      mvirm(1,3)=mvirm(1,3)+tm3
      mvirm(2,1)=mvirm(1,2)
      mvirm(2,2)=mvirm(2,2)+tm4
      mvirm(2,3)=mvirm(2,3)+tm5
      mvirm(3,1)=mvirm(1,3)
      mvirm(3,2)=mvirm(2,3)
      mvirm(3,3)=mvirm(3,3)+tm6


      return
      end
