      subroutine torsion
c
c------------------------------------------------------------------------
c
c Calculate energy and forces due to torsional angle potential
c
c torsional angles are defined by 4 backbone atoms:
c
c                                    Cj+1
c                                    /^
c                                   / bj+1
c                           phi(j) /
c                      Cj-1-------Cj          this is 0 degrees
c               ^     /     ^
c               bj-1 /      bj
c                   /
c                  Cj-2
c
c Torsional angles and potentials are calculated for the 2nd through
c n-1 the bonds.  Since the terminal methyl groups are assumed spherical
c there is no need to include them.
c
c------------------------------------------------------------------------

      implicit none

c     +---------------+
c     | Include Files |
c     +---------------+

#include "ppmdparam1.h"

#include "bonds.h"
#include "energy.h"
#include "forces.h"
#include "paramsr.h"
#include "sizes.h"

#include "gencoords.h"
#include "steps.h"
#include "times.h"
#include "deiktes.h"

c#include "pentrnt.h"
c#include "bonds_pen.h"
#include "which.h"

c     +-----------------+
c     | Local Variables |
c     +-----------------+

      integer ich,iprim
      integer i,l,k,dnun,dnunm1,j,m
      integer itmp

      real*8 inblen(maxatmc,maxchain)
      real*8 inIbxbI(0:maxatmc,maxchain)
      real*8 cosphi(maxatmc,maxchain),pairp,rot,
     1       gradm2x,gradm2y,gradm2z,gradm1x,gradm1y,gradm1z,
     2       gradx,grady,gradz,gradp1x,gradp1y,gradp1z,
     3       dudcos(maxatmc),ssum
      real*8 univx(3),univy(3),univz(3),univxb(3,maxatmc,maxchain),
     1       univyb(3,maxatmc,maxchain),univzb(3,maxatmc,maxchain)
      real*8 vxb(3),vyb(3),vzb(3),vxbp(3),vybp(3),vzbp(3),
     1       vxbn(3),vybn(3),vzbn(3),v(3),vp(3)

      real*8 tmp1,tmp2,tmp3,tmp4
	logical condition


c
c------------------------------------------------------------------------
c
c

	itmp=0
	eptasum = 0.0d0

      univx(1)=1.0d0
      univx(2)=0.0d0
      univx(3)=0.0d0

      univy(1)=0.0d0
      univy(2)=1.0d0
      univy(3)=0.0d0

      univz(1)=0.0d0
      univz(2)=0.0d0
      univz(3)=1.0d0

      pairp = 0.0d0
      dudcos(1)=0.d0
      dudcos(maxatmc)=0.0d0
	do i=1,maxatmc
	  dudcos(i)=0.0d0
	enddo


      do ich=1,nchains
        do i=1,maxatmc-1

          univxb(1,i,ich)=0.0d0
          univxb(2,i,ich)=-b(3,i,ich)
          univxb(3,i,ich)=b(2,i,ich)

          univyb(1,i,ich)=b(3,i,ich)
          univyb(2,i,ich)=0.0d0
          univyb(3,i,ich)=-b(1,i,ich)

          univzb(1,i,ich)=-b(2,i,ich)
          univzb(2,i,ich)=b(1,i,ich)
          univzb(3,i,ich)=0.0d0

        enddo

	do i=1,maxatmc-1
          inblen(i,ich)=1.0d0/blen(i,ich)
	enddo

        b(1,maxatmc,ich)=0.0d0
        b(2,maxatmc,ich)=0.0d0
        b(3,maxatmc,ich)=0.0d0

        inIbxbI(0,ich)=0.0d0

	do i=1,natmc(ich)-2

	  inIbxbI(i,ich)=1.0d0/IbxbI(i,ich)

	enddo

	do i=1,natmc(ich)-3
c          |
c          |                  bi x bi+1     bi+1 x bi+2
c          | cosphi(i+1)= - (-----------).(-----------) 
c          |                 |bi x bi+1|   |bi+1 x bi+2|
c          |
c
           cosphi(i+1,ich)=-(bxb(1,i,ich)*bxb(1,i+1,ich)
     1                     +bxb(2,i,ich)*bxb(2,i+1,ich)
     2                     +bxb(3,i,ich)*bxb(3,i+1,ich))
     3                     *inIbxbI(i,ich)*inIbxbI(i+1,ich)

        tmp1 =  cosphi(i+1,ich) 
      	If ( dAbs(tmp1) .GT. 1.0d0 ) then
       	If ( dAbs(tmp1) - 1.d0 .GE. 1.d-5 ) STOP 'error in torsion'              
        	tmp1 = dSign(1.d0, tmp1)
      	End If
        tmp1 = dacos( tmp1 )

           tmp2 = - b(1,i+2,ich) * bxb(1,i,ich)
     >            - b(2,i+2,ich) * bxb(2,i,ich)
     >            - b(3,i+2,ich) * bxb(3,i,ich)

           tmp3 = dsign(tmp1,tmp2)
	   tmp3 = 180.0d0*tmp3/3.1415927d0

c          | The torsional potential is given by the equation
c          |
c          |  Vtor = kphi*(1-cos3phi)/2
c          |
c          | and with cos3phi=4*cosphi^3-3*cosphi 
c          | we calculate dV/dcosphi
c          |

	   l=dihtype(identmc(i+1,ich),identmc(i+2,ich))
	   rot=rotbar(l)
	   dudcos(i+1)=-0.50d0*rot*(12.0d0*cosphi(i+1,ich)*
     1	   	  cosphi(i+1,ich)-3.0d0)

c
c          | Calculate torsional angle contribution to 
c          | the potential energy
c
           pairp=1.0d0-(4.0d0*cosphi(i+1,ich)*cosphi(i+1,ich)
     1	 	   -3.0d0)*cosphi(i+1,ich)
	     eptasum=eptasum+pairp*0.5d0*rot

        enddo


        do i=1,natmc(ich)-3	
c          | 
c          | Vtor(cosphi(i)) depends on r     ,r     ,r   ,r
c          |                            ~Ci  ~Ci+1  ~Ci+2  ~Ci+3
c          |
c          | so we have to calculate the appropriate derivatives

           do j=1,3

	      vxb(j)=univxb(j,i+1,ich)
	      vxbp(j)=univxb(j,i,ich)
	      vxbn(j)=univxb(j,i+2,ich)

	      vyb(j)=univyb(j,i+1,ich)
	      vybp(j)=univyb(j,i,ich)
	      vybn(j)=univyb(j,i+2,ich)

	      vzb(j)=univzb(j,i+1,ich)
	      vzbp(j)=univzb(j,i,ich)
	      vzbn(j)=univzb(j,i+2,ich)

 	      v(j)=-vvect(j,i+1,ich)
	      vp(j)=-vvect(j,i,ich)	
 
           enddo
c
c          | dV(cosphi(i))/drCi
c                            ~
           gradm2x=dudcos(i+1)*inIbxbI(i,ich)
     1             *inblen(i,ich)*
     1	          (vxb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2             vxb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+ 
     3             vxb(3)*(v(3)+cosphi(i+1,ich)*vp(3))) 

           gradm2y=dudcos(i+1)*inIbxbI(i,ich)
     1             *inblen(i,ich)*
     1          (vyb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+ 
     2          vyb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+ 
     3          vyb(3)*(v(3)+cosphi(i+1,ich)*vp(3)))

           gradm2z=dudcos(i+1)*inIbxbI(i,ich)
     1            *inblen(i,ich)*
     1          (vzb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+  
     2          vzb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+ 
     3          vzb(3)*(v(3)+cosphi(i+1,ich)*vp(3))) 

c
c          | dV(cosphi(i))/drCi+1
c                            ~

           gradm1x=dudcos(i+1)*(-inIbxbI(i,ich)*
     1             (inblen(i,ich)*
     1             (vxb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2             vxb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3             vxb(3)*(v(3)+cosphi(i+1,ich)*vp(3)))+
     x             inblen(i+1,ich)*
     4             (vxbp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     5             vxbp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     6             vxbp(3)*(v(3)+cosphi(i+1,ich)*vp(3))))+
     7             inIbxbI(i+1,ich)*inblen(i+1,ich)*
     8	           (vxbn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9             vxbn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9             vxbn(3)*(vp(3)+cosphi(i+1,ich)*v(3))))

           gradm1y=dudcos(i+1)*(-inIbxbI(i,ich)*
     1             (inblen(i,ich)*
     1             (vyb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2             vyb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3             vyb(3)*(v(3)+cosphi(i+1,ich)*vp(3)))+
     x             inblen(i+1,ich)*
     4             (vybp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     5             vybp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     6             vybp(3)*(v(3)+cosphi(i+1,ich)*vp(3))))+
     7             inIbxbI(i+1,ich)*inblen(i+1,ich)*
     8             (vybn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9             vybn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9             vybn(3)*(vp(3)+cosphi(i+1,ich)*v(3))))
    
    
           gradm1z=dudcos(i+1)*(-inIbxbI(i,ich)*
     1             (inblen(i,ich)*
     1             (vzb(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2             vzb(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3             vzb(3)*(v(3)+cosphi(i+1,ich)*vp(3)))+
     x             inblen(i+1,ich)*
     4             (vzbp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     5             vzbp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     6             vzbp(3)*(v(3)+cosphi(i+1,ich)*vp(3))))+
     7             inIbxbI(i+1,ich)*inblen(i+1,ich)*
     8             (vzbn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9             vzbn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9             vzbn(3)*(vp(3)+cosphi(i+1,ich)*v(3))))

c
c          | dV(cosphi(i))/drCi+2
c                  ~
           gradx=dudcos(i+1)*(inIbxbI(i,ich)*inblen(i+1,ich)*
     1           (vxbp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2           vxbp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3           vxbp(3)*(v(3)+cosphi(i+1,ich)*vp(3)))-
     4           inIbxbI(i+1,ich)*(inblen(i+1,ich)* 
     5	         (vxbn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     6           vxbn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     7           vxbn(3)*(vp(3)+cosphi(i+1,ich)*v(3)))+
     x           inblen(i+2,ich)*
     8           (vxb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9           vxb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9           vxb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))))

           grady=dudcos(i+1)*(inIbxbI(i,ich)*inblen(i+1,ich)*
     1           (vybp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2           vybp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3           vybp(3)*(v(3)+cosphi(i+1,ich)*vp(3)))-
     4           inIbxbI(i+1,ich)*(inblen(i+1,ich)*
     5           (vybn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     6           vybn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     7           vybn(3)*(vp(3)+cosphi(i+1,ich)*v(3)))+
     x           inblen(i+2,ich)*
     8           (vyb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9           vyb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9           vyb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))))
       
           gradz=dudcos(i+1)*(inIbxbI(i,ich)*inblen(i+1,ich)*
     1           (vzbp(1)*(v(1)+cosphi(i+1,ich)*vp(1))+
     2           vzbp(2)*(v(2)+cosphi(i+1,ich)*vp(2))+
     3           vzbp(3)*(v(3)+cosphi(i+1,ich)*vp(3)))-
     4           inIbxbI(i+1,ich)*(inblen(i+1,ich)*
     5           (vzbn(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     6           vzbn(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     7           vzbn(3)*(vp(3)+cosphi(i+1,ich)*v(3)))+
     x           inblen(i+2,ich)*
     8           (vzb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     9           vzb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     9           vzb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))))

c
c          | dV(cosphi(i))/drCi+3
c                       ~
           gradp1x=dudcos(i+1)*inIbxbI(i+1,ich)*inblen(i+2,ich)*
     1             (vxb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     2             vxb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     3             vxb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))
		       
           gradp1y=dudcos(i+1)*inIbxbI(i+1,ich)*inblen(i+2,ich)*
     1             (vyb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     2             vyb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     3             vyb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))
					     
           gradp1z=dudcos(i+1)*inIbxbI(i+1,ich)*inblen(i+2,ich)*
     1             (vzb(1)*(vp(1)+cosphi(i+1,ich)*v(1))+
     2             vzb(2)*(vp(2)+cosphi(i+1,ich)*v(2))+
     3             vzb(3)*(vp(3)+cosphi(i+1,ich)*v(3)))
     
c
c         | Find the index of primary atom to add the force due to
c         | potential from phi(i)
c	POLYDIMETHYLSILAETHYLENE


          fxmc(i,ich)=fxmc(i,ich)-gradm2x
          fymc(i,ich)=fymc(i,ich)-gradm2y
          fzmc(i,ich)=fzmc(i,ich)-gradm2z
c
c
          fxmc(i+1,ich)=fxmc(i+1,ich)-gradm1x
          fymc(i+1,ich)=fymc(i+1,ich)-gradm1y
          fzmc(i+1,ich)=fzmc(i+1,ich)-gradm1z
c
c
c
          fxmc(i+2,ich)=fxmc(i+2,ich)-gradx
          fymc(i+2,ich)=fymc(i+2,ich)-grady
          fzmc(i+2,ich)=fzmc(i+2,ich)-gradz
c
c
c
          fxmc(i+3,ich)=fxmc(i+3,ich)-gradp1x
          fymc(i+3,ich)=fymc(i+3,ich)-gradp1y
          fzmc(i+3,ich)=fzmc(i+3,ich)-gradp1z
c
      enddo
c

      enddo
      RETURN
      END
