      program ppmd
c
c  Atactic poly(propylene) Molecular Dynamics program.
c
c
      implicit none

#include "ppmdparam1.h"
#include "ppmdparam2.h"
#include "ppmdparam3.h"

      integer lnblnk
      integer idummy,i,j,anumber
      integer io
      logical penpresent

      character*9  tmp1,tmp2
      character*40  inpfil,filnam
      character*1  ffc
      character*70 message

      real*8 stelp
	real TimeInSecs,second
	external second
c
c--------------------------------------------------------------------
c
c
#include "averages.h"
#include "box.h"
#include "box2.h"
c #include "chirpen.h"
#include "control.h"
#include "conv.h"
#include "derivs.h"
#include "errors.h"
#include "forces.h"
#include "initial.h"
#include "listi.h"
#include "listr.h"
#include "paramsr.h"
#include "port1.h"
#include "port2.h"
#include "scaling.h"
#include "scalpos.h"
#include "shape.h"
#include "sizes.h"
#include "steps.h"
#include "strings.h"
#include "switches.h"
#include "times.h"
#include "which.h"
#include "zero.h"

c
c The following common blocks aren't used in ppmd1, but include them
c here so they will remain "active" throughout program execution
c
#include "bonds.h"
#include "constnts.h"
#include "coeff.h"
#include "doros.h"
#include "expene.h"
#include "geom.h"
#include "index.h"
#include "paramsi.h"
#include "parents.h"
#include "posit.h"
#include "potprm.h"
#include "props.h"
#include "tables.h"

#include "energy.h"

c #include "pentrnt.h"
c #include "listpen.h"
c #include "pen_old.h"
c #include "bonds_pen.h"

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	
      nstep=0
      kount=0
      nequil=0
      nfcalc=0
      nfailed=0
      nminmz=0
      nzero=0
      image=0
      rdelta=1.0d0


c ---- MLG new

	open(unit=20,file='properties')
	open(unit=30,file='molnrg')
	open(unit=31,file='extnrg')

      CALL initialize

c ---- end MLG new

c
c      Get input datafile name from command line or default to 
c      'mdata.dat', then read in run data set, and set up all kinds 
c      of stuff in ppinit.
c
c       CALL getarg(1,inpfil)
	inpfil='mdatNPT'
c    
c  Initialize
c
      CALL ppinit(inpfil)

c
c     Define various run-time constants
c     tmid = ndof*kT/2 = mvv/2
c
      tmid=ndof*tstar/2.0d0
	Do i=1,nchains
	  tmid = tmid/(3*natoms(i))
	End Do
	tmid = tmid*(9.0d0)

c   trajdel = time step for saving atomic trajectories      

      trajdel=1.0d-12/timscl

      ffc=char(12)
c
c     Initialize control variables and flags.
c
c     See if velocity scaling is to be done.
c     elapse*pstimscl = current simulation time (ps)
c     tvscal*pstimscl = time when to stop velocity scaling (ps)
c
      if(elapse.lt.tvscal)then
            scaling=.true.
            tscal3=elapse
            tscal4=elapse+tvscalon
         else
            scaling=.false.
            tscal3=tvscal
      endif
c
c     Make sure you have not specified velocity scaling and the Nose
c     thermostat.
c
      if(scaling.and.nose) then
         write(10,20)
         stop 'U cant specify velocity rescal. and Nose therm.'
      endif
   20 format(/,2x,'*** You can not specify velocity rescaling and ',
     x       'Nose thermostat ***')
c
c     I (KFM?) changed this to write out initial conditions
c              (during the first time ppmd3 is called)
c
      tpr1=elapse
      tprwrite1=elapse
      tsav1=elapse+tsav
      equil=.false.
c
c     tequil*pstimscl = time to equilibrium (ps)
c
      if(tequil.lt.elapse) then
         equil=.true.
         write(10,54)elapse*pstimscl
      endif
 54   format(2x,'** System equilibrated at time',f9.4,
     x       ', accumulating averages now **')
c
c     tdampstrt*pstimscl = time to start damping 
c     tdampend*pstimscl = time to stop damping 
c
      damping=.false.
      if(tdampstrt.le.elapse.and.tdampend.gt.elapse) then
          damping=.true.
          write(10,35) elapse*pstimscl
      endif
 35   format(2x,'Wall damping started at time ',f9.4)
c
c     If errtol <= 0 then disable error checking and use a constant 
c     time step. Errtol is the allowed error in i
c     nerrck = wait period before trying to increase timestep
c     nerror = wait period before checking error in accelerations
c
      if(errtol.le.0.0d0)then
            nerror=int( (ttot-elapse)/delta+1.5d0 )
c           nerror=(ttot-elapse)/delta+1.5d0
            nerrck=nerror
         else
c
c           No error checking for the first 10 time steps to allow the 
c           predictor series to stabilize
c
            nerror=10
            nerrck=10
      endif
      stelp=elapse
      if(tsav1.le.ttot) then
            write(10,39)savfil(1:lnblnk(savfil))
         else
            write(10,51)
      endif
      if(finsav) then
            write(10,36)posfil(1:lnblnk(posfil)),
     x                  crayid(1:lnblnk(crayid))
         else
            write(10,52)
      endif
 39   format(2x,'Intermediate atom pos and vel will be saved in files ',
     x       a)
 36   format(2x,'Final run info save file = ',a,/,2x,
     x       'Final run averages save file = ',a,'fav')
 51   format(/,2x,'No intermediate save files')
 52   format(2x,'No save of final arrays')
c
c     open .prn file if desired.
c
      if(tprwrite1.le.ttot.and.tprwrite.gt.0)then
            if(prnbin)then
                  open(unit=7,file=outfil,status='unknown',
     1            form='unformatted')
               else
                  open(unit=7,file=outfil,status='unknown',
     1                 form='formatted')
            endif
            write(10,28)outfil(1:lnblnk(outfil)),prnbin
            open(unit=13,file=crayid(1:lnblnk(crayid))//'E',
     x           status='unknown',form='formatted')
         else
            write(10,50)
      endif
 28   format(2x,'Output file = ',a,/,2x,'Output in binary format? ',l4)
 50   format(2x,'No output file')
c
c     Open first trajectory dump file if desired
c     - all atom positions & velocities are dumped for later analysis.
c
      if(trajdmp) then

            trajtme=elapse

            trajtme=elapse+trajdel
            ntraj=1
            ntrajfile=1


            write(10,70) trajdel/1.0d-15*timscl,
     x                   trajfil(1:lnblnk(trajfil))//'001'
            write(10,71) trajdel/1.0d-15*timscl,
     x                   disfil(1:lnblnk(disfil))//'001'
         else
            write(10,60)
      endif
 60   format(2x,'No saving of atomic trajectories')
 70   format(2x,'Atom trajectories saved every ',f10.4,' 
     x        fs in file: ',a)
 71   format(2x,'Vectors of dispacement saved every ',
     x       f10.4,' fs in file: ',a)
c
c     Define backup file names
c
      bakfil=crayid(1:lnblnk(crayid))//'bak'
c
c     Check to see if the file crayid//'ave' exists.  If it does, then
c     read in the previously accumulated averaging information.
c
      if(restrt.and.avexist) then
          write(10,75) avefil(1:lnblnk(avefil))
          open(unit=8,file=avefil,status='old')
          read(8,*) estep
		read(8,*) nequil
		read(8,*) avtote
          read(8,*) avtesq
		read(8,*) avpote
		read(8,*) avpesq
		read(8,*) avpec
		read(8,*) avpecsq
		read(8,*) avpebl
		read(8,*) avplsq
          read(8,*) avpeba
		read(8,*) avpasq
		read(8,*) avpenb
		read(8,*) avpnsq
		read(8,*) avpeta
          read(8,*) avptsq
		read(8,*) avmpint
		read(8,*) avmpsq
		read(8,*) avpic
		read(8,*) avpicsq
          read(8,*) avmstrss
		read(8,*) avmstrssq
		read(8,*) avdens
		read(8,*) avdnsq
          read(8,*) avekin
		read(8,*) avkesq
		read(8,*) avewk
		read(8,*) avwksq
          read(8,*) avesp
		read(8,*) avspsq
		read(8,*) avhame
          read(8,*) avhasq
		read(8,*) avtramp
          close(unit=8)
      endif
 75   format(/,2x,'Reading in old averaging information from ',a)
c
c     Set temporary file to save averaging information in
c
      avefil=crayid(1:lnblnk(crayid))//'ave'
c
c     ffc = ^L (forces page break in output)
c
      write(10,42) ffc
 42   format(a1,/,2x,'Time',6x,'Temp',5x,'Urans',3x,'Potene',
     x      5x,'Totene',5x,'Tails',5x,'Hamil',4x,'Dens.',4x,'Peevee',
     x      7x,'Int. P',7x,'P.tails',6x,'P.total',6x,'Pair',6x,
     x      'B.ang.',4x,'B.len.',3x,'Torsion',5x,'Nose',' Trans')
c


      call ppmd2
c
c     OK, we're done at last.
c
c     Dump everything important to posfil for future restart if desired.
c     Since so much stuff is being dumped, save it in binary form to
c     conserve disk space.  Also, remove time step info from taylor
c     series elements since next run may have different time step.
c     In addition, save averaging information, by a call to savave,
c     in case we want to use it in the next run.
c
      write(6,990)
  990 format(2x,'*** Compute final averages of properties ***')
      if(finsav) then
          CALL wrterst(posfil)
          filnam=crayid(1:lnblnk(crayid))//'fav'
          CALL savave(filnam)
      endif
c
c     Calculate and print property averages if information on them was
c     collected.
c     estep = length of time spent averaging (avecalc.F)
c     kount = # of times Verlet called
c     nequil = # of calls to avecalc.F
c     nrstep = # of times time step was reduced
c     nfcalc = # of force calcs
c     nstep = # of time steps
c
      write(10,48) ffc,nstep,nequil,pstimscl*estep,
     x             (elapse-stelp)*pstimscl/nstep,
     x             (elapse-stelp)*pstimscl/nstep*1.0d+3,
     x             float(nfcalc)/float(nstep),nrstep,
c     x			 nstep/kount,nzero,
     x             image
      if(constraint) write(10,76) nminmz,nstep/max(nminmz,1)
      if(nfailed.ne.0) write(10,67)nfailed,
     x                 float(nfailed)/float(nstep)*100.0
      if(equil) then
            call prntiave(elapse)
         else
            write(10,37)
      endif
 48   format(a1,/,2x,'Total time steps = ',i10,/,2x,
     x        i6,' steps (',f7.3,' picoseconds)',
     x        ' used for averaging',/,2x,
     x        'Average run time step = ',f8.6,' ps  ( ',
     x        f8.6,' fs )',/,2x,
     x        'Average force calculations per step = ',f7.4,/,2x,
     x        'Number of times the time step was reduced = ',i6,/,2x,
     x        'Average steps between list updates = ',i6,/,2x,
     x        'Number of times net momentum was re-zeroed = ',i6,/,2x,
     x        'Number of steps where Min. Image Convention was ',
     x        'violated = ',i6)
 76   format(2x,'Number of penalty minimizations = ',i10,/,
     x       2x,'Average steps between penalty minimizations = ',i10)
 67   format(2x,'There were ',i3,' steps (',f6.4,'%) taken with',
     x         ' unacceptable error.')
 37   format(2x,'No property averaging information was collected in',
     x       ' this run.',/,2x,'No property averages calculated.')
c
c     It's Miller time!  Close trajectory file, delete backup files,
c     write final coordinates, calculate how much money we just
c     wasted and how long it took us to do it.
c
      if(trajdmp) close(unit=9,status='keep')

          open(unit=8,file=bakfil)
          close(unit=8,status='delete')
          open(unit=8,file=avefil)
          close(unit=8,status='delete')
          CALL wrterst(bakfil)
          CALL savave(avefil)

      bakfil=crayid(1:lnblnk(crayid))//'xyz'
      open(unit=8,file=bakfil,status='unknown',form='formatted')
      idummy=2
	write(8,*) nchains,nspec(0)
	Do i=1,nspec(0)
		Read(8,*) nspec(i), McNat(i), SuNat(i)
	End Do


      do 620 i=1,3
         write(8,630) (h(i,j),j=1,3)
  620 continue
      do 650 j=1,nchains
         do  i=1,natmc(j)
	    write(8,640) identmc(i,j),x0mc(i,j),y0mc(i,j),z0mc(i,j)
	   end do
         do  i=1,natsu(j)
	    write(8,640) identsu(i,j),x0su(i,j),y0su(i,j),z0su(i,j)
	   end do
  650 continue

 601  format(1x,'''',a,'''',2x,i6,1x,i6,1x,d13.6)
  600 format(i2,1x,'''',a,'''')
  610 format(i7,2i8)
  630 format(3(1x,d23.16))
  640 format(i3,1x,3(1x,d23.16))

      close(unit=8)
      close(unit=13)
c
c
      io=10
	close(unit=97)
	close(unit=20)
	close(unit=30)
	close(unit=31)
c
      write(6,*) 'run is over' 
      stop
      end
