      subroutine ppmd2
c
c-----------------------------------------------------------------------
c
c  Atactic poly(propylene) Molecular Dynamics program.
c
c-----------------------------------------------------------------------
c
      implicit none

#include "ppmdparam1.h"
#include "ppmdparam2.h"
#include "ppmdparam3.h"

      real*8 uordr,smallpnum

      parameter (uordr=0.16666667d0)
      parameter (smallpnum=1.0d-9)
c
      integer i,j,ii,ij,jj,ll,k,nx,ny,nz

      logical update,justmade,updatepen,justmadepen,penalty,chkerr
     >      , zerotrns,calcbl, mike, moresteps
c     >      , zerotrns,calcbl,first, mike, moresteps
c
      real*8 delsqh,delsqhix,delsqhiy,delsqhiz
     >     , delsqh2x,delsqh2y,delsqh2z,rngex,rngey,rngez
      real*8 testval,x,y,z,hdt(3,3)
	external updatepen
c--------------------------------------------------------------------

	logical savefirst

#include "box.h"
#include "box2.h"
#include "control.h"
#include "conv.h"
#include "derivs.h"
#include "energy.h"
#include "errors.h"
#include "forces.h"
#include "initial.h"
#include "port1.h"
#include "port2.h"
#include "potprm.h"
#include "print.h"
#include "props.h"
#include "scaling.h"
#include "scalpos.h"
#include "shape.h"
#include "sizes.h"
#include "steps.h"
#include "strings.h"
#include "switches.h"
#include "times.h"
#include "zero.h"

c #include "pentrnt.h"

c recently added by S&C
#include "listi.h"
c #include "listpen.h"
#include "posit.h"

c following is for printing and scaling fx
#include "which.h"

c debugging energy/force check
#include "mike.h"
#include "paramsr.h"
c	integer mistbli(maxbonds),mistblj(maxbonds)
	real*8 deli,delisq,delisq2,delisq2x,delisq2y,delisq2z
C	DEBUGGING
	logical condition
	integer number

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c
c     | Okay! Let's get down and do some simulating!
c     | scnd = T if this is the second time through. This happens only 
c     | when constraints are incorporated.
c
      write(6,995)
 995  format(2x,'*** Start simulation! ***')
      
      prinder=.false.
      justmade = .false.
      justmadepen = .false.
      mike = .false.

 100  continue
      scnd=.false.
c
c     | Save current values of Taylor series for all degrees of freedom.
c     | This is done in case you need to repeat the predictor with a
c     | reduced timestep.
c
c      CALL saveold
c
 101  continue                  

c     +---------------+
c     |  New timestep |
c     +---------------+
c     |
c     |  rdelta is set to 1 the first time through (ppmd1)
c     |  if rdelta .ne. 1, all higher derivatives of x0, hl0, sp0 are
c     |_ changed to the appropriate internal coords.
c

      if(abs(rdelta-1.0d0).gt.1.0d-8) then
c        CALL chngtime
      endif
c
      delsqh=0.5d0*delsq
c
c     +-----------------------+
c     | Predictor - Corrector |
c     +-----------------------+
c     |
c     | Fifth order (six value) variable time step 'Gear' predictor.
c     | [C.W. Gear; chapter 9 of "Numerical Initial Value Problems in
c     | Ordinary Differntial Equations", (Prentice-Hall), 1971.]
c     |
c     | Predict values at next timestep
c     |_The predictr also initializes fx,fy,fz to zero
c
c     write(10,200)
 200  format(2x,'Predict new coordinates')

      CALL predictr
c
c     | Calculate random useful things  
c     | - this should only occur once at the beginning of ppmd2 for
c     | systems that don't change size or shape !!!
c     | hd is times sqrt(mH)*L
c
      if(uniform) then
            do 105 i=1,3
               do 105 j=1,3
                  h(j,i)=hinit(j,i)*hl0
                  hd(j,i)=hinit(j,i)*hl1
  105       continue
      endif

      CALL trnspse(h,ht)
      CALL mxm(ht,3,h,3,g,3)
      g11=g(1,1)
      g22=g(2,2)
      g33=g(3,3)
      g12t=2.0d0*g(1,2)
      g13t=2.0d0*g(1,3)
      g23t=2.0d0*g(2,3)
c 
c     | Get inverse(hi) and determinant(vol) of h
c
      CALL matinv(h,hi,vol)
c
c     | scnd = F first time through
c     | scnd = T second time through, you do this only if imposing 
c     |          bond length constraints
c     |
c     | pimat  = particle momentum matrix
c     | virmat = Virial matrix
c
      if(.not.scnd) then   
         heatp=1.0d0
         heatw=1.0d0
         do ii=1,3
            do ij=1,3
               pimat(ij,ii)=0.0d0
               mvirm(ij,ii)=0.0d0
            end do
         end do
c
c        | hhh=(h')**2  times mH/(L^2)
c
         CALL trnspse(hd,hdt)
         CALL mxm(hdt,3,hd,3,hhh,3)
         do 107 ii=1,3
            do 107 ij=1,3
               hhh(ij,ii)=hhh(ij,ii)/delsq
 107     continue
         CALL trnspse(hi,hit)
      endif           
c
c     | Formation of H center positions and calculation of
c     | Current real space image displacements and other
c     | useful things
c
        CALL formh 

c     +---------------------------+
c     | Make verlet neighbor list |
c     +---------------------------+
c     |
c     | Neighbor list is made up whenever the largest displacement since
c     | the last update is greater than half the skin thickness.
c     |
c     | Test to see if a list update is required. Use value of 50%
c     | of the skin thickness (stored in  skntst as the square). 
c     | Also update list if we just updated the list and the
c     |_last integration step gave too big an error.
c
      testval=skntst
 104  if(update(testval).or.(retry.and.justmade)) then
         CALL verlet
         justmade    = .true.
        write(6,222)elapse*pstimscl

      else
         justmade=.false.
      endif
 222  format(2x,'Time = ',f10.5,' Remake neighbor list')

c
c     | Force calculation based on predicted positions and derivatives.
c     | Zero averages first:
c     |     avbl = average bond length
c     |     avblsq = avbl**2
c
      do i=1,3
        avbl(i)=0.0d0
        avblsq(i)=0.0d0
      end do

      if(.not.scnd) then
c
c        | nfcalc = number of force calculations
c
         nfcalc=nfcalc+1
c
c        | Zero all energy contributions, then calculate new forces.
c        | epnbsum = pairwise nonbonded potential energy
c        | epblsum = bond potential energy
c        | epbasum = bond angle potential energy
c        | eptasum = torsion angle potential energy
c        | epfixsum= Fixman potential energy
c        | eppensum= penetrant-polymer nonbonded potential energy
c
         epblsum=0.0d0
         epnbsum=0.0d0
         epbasum=0.0d0
         eptasum=0.0d0
         epfixsum=0.0d0
         eppensum=0.0d0
         epblpen = 0.0d0
         epbapen = 0.0d0
         eptapen = 0.0d0
         epnbpen = 0.0d0
         epchirsum=0.0d0


         CALL calcfrcs

c	 hamil = trans + epnbsum + epbasum + eptasum ! new revision scheme
c	 if(nstep.eq.1) hamil0=hamil
c	 adapt, ad2 and adi will be included in errors.h
	
	else
	  write(10,*)'ppmd2: found scnd=true during supposed first loop'
        write(10,*)'       (where calcfrcs would have been called)'
      endif


c     +-----------------------------------+
c     | Determine local integration error |
c     +-----------------------------------+
c     |
c     | f's are the differences of accelerations in units of L.
c     |_Replace accelerations fx by diff's in accelerations x2 - fx
c

      delsqh2x=delsqh/(h(1,1))
      delsqh2y=delsqh/(h(2,2))
      delsqh2z=delsqh/(h(3,3))
	deli=1.0d0/delta
	delisq=deli*deli
	delisq2=2.0d0*delisq
	delisq2x=delisq2*h(1,1)
	delisq2y=delisq2*h(2,2)
	delisq2z=delisq2*h(3,3)

      do j=1,nchains
         do i=1,natmc(j)
           fxmc(i,j)=x2mc(i,j)-fxmc(i,j)*delsqh2x
           fymc(i,j)=y2mc(i,j)-fymc(i,j)*delsqh2y
           fzmc(i,j)=z2mc(i,j)-fzmc(i,j)*delsqh2z
         end do
         do i=1,natsu(j)
           fxsu(i,j)=x2su(i,j)-fxsu(i,j)*delsqh2x
           fysu(i,j)=y2su(i,j)-fysu(i,j)*delsqh2y
           fzsu(i,j)=z2su(i,j)-fzsu(i,j)*delsqh2z
         end do
      end do

      if(move) then
         if(uniform) then
            hldd=hl2-hldd*delsqh
         endif
      endif

      if(nose) spd=sp2-spd*delsqh
c
c     | First nerror steps don't do error checking to allow the elements
c     | of the taylor series to stabilize.  Therefore, the delta given in
c     | the input file should be quite small to prevent truncation error
c     | during this time. Nerror is initially set to ten, but gets 
c     | updated every so often.
c
      if(nstep.gt.nerror) then
c
c        | Check error and if too big repeat last step with new smaller
c        | timestep (calculated in chkerr). Routine returns with
c        | retry = F and chkerr = F if the timestep is to be increased.
c
c         if(chkerr()) then
c            write(10,450)
c            nrstep=nrstep+1
c            goto 101  ! comment out when new delta revision scheme applied
c         endif
      endif

c 450  format(2x,'Error not OK, repeating prediction with smaller step')
c
c     | Error was OK. First, "Gear" corrector.
c
c     write(10,430)
 430  format(2x,'Correcting prediction')

      CALL correctr

c     |
c     | Save old step for property averages, etc.
c     |
      odelta=delta
c	COMMENT following FOR DEBUGGING
c	 or when new delta revision scheme applied
      rdelta=1.0d0 
c
#ifdef NOTHING
      IF(CONSTRAINT) THEN
c
c        | Re-calculate some things which the corrector may have altered
c
         if(uniform)then
            do 115 ii=1,3
               do 115 ij=1,3
                  h(ij,ii)=hinit(ij,ii)*hl0
                  hd(ij,ii)=hinit(ij,ii)*hl1
  115       continue
         endif
         CALL trnspse(h,ht)
         CALL mxm(ht,3,h,3,g,3)
         g11=g(1,1)
         g22=g(2,2)
         g33=g(3,3)
         g12t=2.0d0*g(1,2)
         g13t=2.0d0*g(1,3)
         g23t=2.0d0*g(2,3)
         CALL matinv(h,hi,vol) 
 
c        | --    NEW CODE from S&C 12-1-95
c        |
c        | Compute real-space coordinates of box images of primary atoms,
c        | and place them in rx, ry, rz

         do 112 i=1,27
              rxl(i)=h(1,1)*xl(i)+h(1,2)*yl(i)+h(1,3)*zl(i)
              ryl(i)=h(2,1)*xl(i)+h(2,2)*yl(i)+h(2,3)*zl(i)
              rzl(i)=h(3,1)*xl(i)+h(3,2)*yl(i)+h(3,3)*zl(i)
 112     continue

         do j = 1, nchains
            do i = 1,nprim

               x = x0(i,j)
               y = y0(i,j)
               z = z0(i,j)

               nx = x + x
               x0(i,j) = x - nx
               ny = y + y
               y0(i,j) = y - ny
               nz = z + z
               z0(i,j) = z - nz

               rx(i,j)=h(1,1)*x0(i,j)+h(1,2)*y0(i,j)
     $                +h(1,3)*z0(i,j)
               ry(i,j)=h(2,1)*x0(i,j)+h(2,2)*y0(i,j)
     $                +h(2,3)*z0(i,j)
               rz(i,j)=h(3,1)*x0(i,j)+h(3,2)*y0(i,j)
     $                +h(3,3)*z0(i,j)

            end do

            do k=1,natmc(j)+natsu(j)-1
               i=listbli(k)
               jj=listblj(k)
               x=x0(i,j)-x0(jj,j)
               y=y0(i,j)-y0(jj,j)
               z=z0(i,j)-z0(jj,j)
               nx=x+x
               ny=y+y
               nz=z+z
               mibl(k,j)=9*nx+3*ny+nz+14
         if ((mibl(k,j).lt.1).or.(mibl(k,j).gt.28)) then
           write(6,*) 'ppmd2 :', k,j,mibl(k,j)
	   write(6,*) x,y,z
	   write(6,*) i,jj,nstep
        endif

            end do
         end do


         do j = 1, npen
           do i = 1,natompen

             x = xpen0(i,j)
             y = ypen0(i,j)
             z = zpen0(i,j)

             nx = x + x
             xpen0(i,j) = x - nx
             ny = y + y
             ypen0(i,j) = y - ny
             nz = z + z
             zpen0(i,j) = z - nz

             rxpen(i,j) = h(1,1) * xpen0(i,j)
     >                           + h(1,2) * ypen0(i,j)
     >                           + h(1,3) * zpen0(i,j)
             rypen(i,j) = h(2,1) * xpen0(i,j)
     >                           + h(2,2) * ypen0(i,j)
     >                           + h(2,3) * zpen0(i,j)
             rzpen(i,j) = h(3,1) * xpen0(i,j)
     >                           + h(3,2) * ypen0(i,j)
     >                           + h(3,3) * zpen0(i,j)

           end do

           do k=1,natompen-1
             i=k
             jj=k+1
             x=xpen0(i,j)-xpen0(jj,j)
             y=ypen0(i,j)-ypen0(jj,j)
             z=zpen0(i,j)-zpen0(jj,j)
             nx=x+x
             ny=y+y
             nz=z+z
             mibl_pen(k,j)=9*nx+3*ny+nz+14
           end do

         end do
   
c
c        | Check that constraints are still being satisfied.  If not,
c        | then nudge atom positions and velocities back where they
c        | should be.  Since this is a discontinuity, delay the next 
c        | error check a bit and reduce the timestep if the variable 
c        | step integrator is in use.
c        | penalty = T if either the positions or velocities of the 
c        |        atoms were nudged.
c        | Phicall = tolerance for correcting the bond lengths to 
c        |        phitol
c        | Psicall = tolerance for correcting the rate of bond length 
c        |          changes to psitol
c        | All these tolerances are read in "ppinit.f".
c
c        if(penalty(phicall,psicall,phitol,psitol)) then
c
C	 DEBUGGING ON AEGEAN: 151200-1  comment out	 
c	if(condition(nstep,0)) then
         if(penalty()) then
            nminmz=nminmz+1
            if(errtol.gt.0.0d0) then
               nerrck=nstep+5
               nerror=nerrck
            endif
         endif
c	endif
c
c        | Reevaluate constraint forces based on CORRECTED positions and
c        | velocities -- improves accuracy of next predictor and 
c        |  reduces number of times penalty minimization is required.
c
         scnd=.true.

c	savefirst=first
c	first=.true.

        CALL formh 
c	first=savefirst

 1004    if(update(testval).or.(retry.and.justmade)) then

            CALL verlet
            justmade    = .true.
            justmadepen = .true.
c            write(10,222)elapse*pstimscl
         else
            justmade=.false.
            if(updatepen(testval).OR.(retry.AND.justmadepen)) then
               CALL makenblst_pen
               justmadepen = .true.
            else
               justmadepen = .false.
            endif
         endif
c
c        | Force calculation based on predicted positions and derivatives.
c
c        | This must be the second time through and you've incorporated
c        | bond length constraints. All that you are doing is making
c        | sure your next prediction is done with the proper bonded
c        | forces, based on after the corrector.
c        | delsqh=0.5d0*delsq
c
         delsqhix=h(1,1)/delsqh
         delsqhiy=h(2,2)/delsqh
         delsqhiz=h(3,3)/delsqh

         do i = 1,nprim
            mikex2(i) = x2(i,1)
         end do

         do j=1,nchains
            do i=1,nprim
               fx(i,j)=x2(i,j)*delsqhix
               fy(i,j)=y2(i,j)*delsqhiy
               fz(i,j)=z2(i,j)*delsqhiz
            end do
         end do
         do j=1,npen   
            do i=1,natompen
               fxpen(i,j)=xpen2(i,j)*delsqhix
               fypen(i,j)=ypen2(i,j)*delsqhiy
               fzpen(i,j)=zpen2(i,j)*delsqhiz
            end do
         end do

         calcbl=.true.

         CALL fixbond(calcbl)

         delsqh2x=delsqh/h(1,1)
         delsqh2y=delsqh/h(2,2)
         delsqh2z=delsqh/h(3,3)
 
         do j=1,nchains
            do i=1,nprim
               x2(i,j)=fx(i,j)*delsqh2x
               y2(i,j)=fy(i,j)*delsqh2y
               z2(i,j)=fz(i,j)*delsqh2z
            end do
         end do
         do j=1,npen   
            do i=1,natompen
               xpen2(i,j)=fxpen(i,j)*delsqh2x
               ypen2(i,j)=fypen(i,j)*delsqh2y
               zpen2(i,j)=fzpen(i,j)*delsqh2z
            end do
         end do
      endif
	END IF(constaint)
	
#endif


 125  continue
c
c     | Check to see if the minimum image criterion is still satisfied.
c     | The cell normals must be larger than 2*rnge(3,3).
c
      rngex=rnge(3,3,1)+dabs(h(1,2))+dabs(h(1,3))
      rngey=rnge(3,3,1)+dabs(h(2,1))+dabs(h(2,3))
      rngez=rnge(3,3,1)+dabs(h(3,1))+dabs(h(3,2))


c
c     | Check that net translational momentum is still zero.  Adjust if
c     | necessary.  This is a discontinuity, so delay next error check.
c     | zerotrns() = T when the translational momentum HAS been reset.
c
      if(zerotrns()) then
         if(condition(nstep,1000)) write(10,65) elapse*pstimscl
         nzero=nzero+1
         if(errtol.gt.0.0d0) then
            nerrck=nstep+10
            nerror=nerrck
         endif
      endif
 65   format(2x,'Time= ',f10.5,' Net drift velocities reset to zero')

c
c
c     | Now calculate new time step if we are due for a change.
c     |
c     | If relerr > 2.66, then rdelta > 1.0 (factor to increase delta by)
c     | Also, wait 5 steps before next increase in delta.
c     | uordr=0.16666667d0 (set in parameter statement)
c     | Nerrck is when the next attempt to increase the time step will 
c     | occur.
c
      if(nstep.gt.nerrck) then

         if(relerr.gt.2.660d0) rdelta=(relerr)**uordr  

            nerrck=nstep+5     
      endif
c

      CALL ppmd3
c
c     | If not done... one more time!
c     | smallpnum = 1.0d-9 (set in parameter statement)
c

      moresteps = ((ttot-elapse).GT.smallpnum)

      if (moresteps) then
         if (elapse+odelta*rdelta.GT.ttot) then
            rdelta=(ttot-elapse)/odelta
         endif
         goto 100
c	 scnd=.false.
      endif

      return
      end
