
      subroutine predictr
c
c  5th order (six value) Gear predictor
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none

#include "ppmdparam1.h"

      integer i,j

#include "derivs.h"
#include "forces.h"
#include "scalpos.h"
#include "sizes.h"
#include "switches.h"

c #include "pentrnt.h"
#include "mike.h"
#include "times.h"

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Predictor for particles (all variables are scaled)
c     e.g. x1=dt*v
c          x2=dt**2*a/2
c          x3=dt**3*b/6
c          x4=dt**4*c/24
c          etc....
c
c     Predictor equation:
c
c      / x0 \     /  1  1  1  1  1  1  \  / x0 \
c      | x1 |     |  0  1  2  3  4  5  |  | x1 |
c      | x2 |  =  |  0  0  1  3  6 10  |  | x2 |
c      | x3 |     |  0  0  0  1  4 10  |  | x3 |
c      | x4 |     |  0  0  0  0  1  5  |  | x4 |
c      \ x5 /     \  0  0  0  0  0  1  /  \ x5 /



       do 1 j=1,nchains
         do i=1,natmc(j)
           fxmc(i,j)=0.0d0
           x0mc(i,j)=x0mc(i,j)+x1mc(i,j)+x2mc(i,j)+
     x               x3mc(i,j)+x4mc(i,j)+x5mc(i,j)
           x1mc(i,j)=x1mc(i,j)+2.0d0*x2mc(i,j)+3.0d0*x3mc(i,j)+
     x               4.0d0*x4mc(i,j)+5.0d0*x5mc(i,j)
           x2mc(i,j)=x2mc(i,j)+3.0d0*x3mc(i,j)+
     x               6.0d0*x4mc(i,j)+10.0d0*x5mc(i,j)
           x3mc(i,j)=x3mc(i,j)+4.0d0*x4mc(i,j)+10.0d0*x5mc(i,j)
           x4mc(i,j)=x4mc(i,j)+5.0d0*x5mc(i,j)
c
           fymc(i,j)=0.0d0
           y0mc(i,j)=y0mc(i,j)+y1mc(i,j)+y2mc(i,j)+
     x               y3mc(i,j)+y4mc(i,j)+y5mc(i,j)
           y1mc(i,j)=y1mc(i,j)+2.0d0*y2mc(i,j)+3.0d0*y3mc(i,j)+
     x               4.0d0*y4mc(i,j)+5.0d0*y5mc(i,j)
           y2mc(i,j)=y2mc(i,j)+3.0d0*y3mc(i,j)+
     x               6.0d0*y4mc(i,j)+10.0d0*y5mc(i,j)
           y3mc(i,j)=y3mc(i,j)+4.0d0*y4mc(i,j)+10.0d0*y5mc(i,j)
           y4mc(i,j)=y4mc(i,j)+5.0d0*y5mc(i,j)
c
           fzmc(i,j)=0.0d0
           z0mc(i,j)=z0mc(i,j)+z1mc(i,j)+z2mc(i,j)+
     x               z3mc(i,j)+z4mc(i,j)+z5mc(i,j)
           z1mc(i,j)=z1mc(i,j)+2.0d0*z2mc(i,j)+3.0d0*z3mc(i,j)+
     x               4.0d0*z4mc(i,j)+5.0d0*z5mc(i,j)
           z2mc(i,j)=z2mc(i,j)+3.0d0*z3mc(i,j)+6.0d0*z4mc(i,j)+
     x               10.0d0*z5mc(i,j)
           z3mc(i,j)=z3mc(i,j)+4.0d0*z4mc(i,j)+10.0d0*z5mc(i,j)
           z4mc(i,j)=z4mc(i,j)+5.0d0*z5mc(i,j)
	     end do

         do i=1,natsu(j)
           fxsu(i,j)=0.0d0
           x0su(i,j)=x0su(i,j)+x1su(i,j)+x2su(i,j)+
     x               x3su(i,j)+x4su(i,j)+x5su(i,j)
           x1su(i,j)=x1su(i,j)+2.0d0*x2su(i,j)+
     x               3.0d0*x3su(i,j)+4.0d0*x4su(i,j)+5.0d0*x5su(i,j)
           x2su(i,j)=x2su(i,j)+3.0d0*x3su(i,j)+
     x               6.0d0*x4su(i,j)+10.0d0*x5su(i,j)
           x3su(i,j)=x3su(i,j)+4.0d0*x4su(i,j)+10.0d0*x5su(i,j)
           x4su(i,j)=x4su(i,j)+5.0d0*x5su(i,j)
c
           fysu(i,j)=0.0d0
           y0su(i,j)=y0su(i,j)+y1su(i,j)+y2su(i,j)+
     x               y3su(i,j)+y4su(i,j)+y5su(i,j)
           y1su(i,j)=y1su(i,j)+2.0d0*y2su(i,j)+
     x               3.0d0*y3su(i,j)+4.0d0*y4su(i,j)+5.0d0*y5su(i,j)
           y2su(i,j)=y2su(i,j)+3.0d0*y3su(i,j)+
     x               6.0d0*y4su(i,j)+10.0d0*y5su(i,j)
           y3su(i,j)=y3su(i,j)+4.0d0*y4su(i,j)+10.0d0*y5su(i,j)
           y4su(i,j)=y4su(i,j)+5.0d0*y5su(i,j)
c
           fzsu(i,j)=0.0d0
           z0su(i,j)=z0su(i,j)+z1su(i,j)+z2su(i,j)+
     x               z3su(i,j)+z4su(i,j)+z5su(i,j)
           z1su(i,j)=z1su(i,j)+2.0d0*z2su(i,j)+3.0d0*z3su(i,j)+
     x               4.0d0*z4su(i,j)+5.0d0*z5su(i,j)
           z2su(i,j)=z2su(i,j)+3.0d0*z3su(i,j)+
     x               6.0d0*z4su(i,j)+10.0d0*z5su(i,j)
           z3su(i,j)=z3su(i,j)+4.0d0*z4su(i,j)+10.0d0*z5su(i,j)
           z4su(i,j)=z4su(i,j)+5.0d0*z5su(i,j)
	     end do
    1   continue

c
c     Predictor for box walls
c
      if(move) then
         if(uniform) then
c
c          Volume changes only (constant shape)
c
           hldd=0.0d0
           hl0=hl0+hl1+hl2+hl3+hl4+hl5
           hl1=hl1+2.0d0*hl2+3.0d0*hl3+4.0d0*hl4+5.0d0*hl5
           hl2=hl2+3.0d0*hl3+6.0d0*hl4+10.0d0*hl5
           hl3=hl3+4.0d0*hl4+10.0d0*hl5
           hl4=hl4+5.0d0*hl5
         endif
      else
c
c        No volume or shape changes allowed
c
          hldd=0.0d0
          hl1=0.0d0
          hl2=0.0d0
          hl3=0.0d0
          hl4=0.0d0
          hl5=0.0d0
      endif
c
c     Predictor for momentum scaling factor.
c
      spd=0.0d0
c
      if(nose) then
            sp0=sp0+sp1+sp2+sp3+sp4+sp5
            sp1=sp1+2.0d0*sp2+3.0d0*sp3+4.0d0*sp4+5.0d0*sp5
            sp2=sp2+3.0d0*sp3+6.0d0*sp4+10.0d0*sp5
            sp3=sp3+4.0d0*sp4+10.0d0*sp5
            sp4=sp4+5.0d0*sp5
         else
            sp1=0.0d0
            sp2=0.0d0
            sp3=0.0d0
            sp4=0.0d0
            sp5=0.0d0
      endif


      return
      end
