
      subroutine readnew(polyfil)
c
c     Read in the initial coordinates for a polypropylene chain and
c     assign them random maxwell velocities.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      implicit none

#include "ppmdparam1.h"

      integer nci,i,i6,j,iden,iseed,ioR,ioW,ich,iun,iat
      integer lnblnk,idummy,initvel,natold,nunold,n

c     character*7  polyfil
      character*40 polyfil
      character*20 title

      real*8 x,y,z,sumx,sumy,sumz,sumv2,xij1,yij1,zij1,
     x       totmass,gaussdev,heater,ttemp, mwr

      logical ppmdformat, penpresent

#include "box.h"
#include "conv.h"
#include "derivs.h"
#include "initial.h"
#include "paramsr.h"
#include "scaling.h"
#include "scalpos.h"
#include "shape.h"
#include "sizes.h"
#include "switches.h"
#include "times.h"
#include "which.h"

#include "gaussdev.h"
c #include "pentrnt.h"

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      open(unit=8,file=polyfil,status='old',form='formatted')
c
c     nun = degree of polymerization
c     natoms = number of atoms per chain
c     nchains = number of chains
c     h = cell dimensions [3x3]
c     ident(1) = H; ident(2) = C; ident(3) = R
c     x0,y0,z0 are the atomic coords (-1/2 to 1/2)L of primary atoms
c     atomic velocity in units of sqrt(epphh/mass(H))
c     delta = time step in units of sqrt(mass(H)/epphh)  !formerly
c                                                        !*sighh also
c     x1,y1,z1 are the atomic velocities*delta in units of boxlengths
c
	if(oldformat) then
        read(8,*) nunold,natold,nchains
c
c	  Check for dimensioning errors
c
        if(nchains.gt.maxchain) then
          write(10,30)
          write(10,*)'     requested ',nchains,' chains'
          stop 'readnew: # of chains dimensioning error'
        endif
	  if(natold.gt.maxatms) then
          write(10,40)
          write(10,*)'     requested ', natold,' atoms'
          write(10,*)'    (requested ', nunold,' repeat units)'
          stop 'readnew: # of atoms dimensioning error'
	  endif

	  Do i=1,nchains
	    natoms(i)=natold
	    natmc(i)=natold/2 +1
	    natsu(i)=natold/2
		nun(i)=natmc(i)-1
	  End Do
	else
	  n = 0
	  Read(8,*) nchains, nspec(0)
c
c	  Check for dimensioning errors
c
        if(nchains.gt.maxchain) then
          write(10,30)
          write(10,*)'     requested ',nchains,' chains'
          stop 'readnew: # of chains dimensioning error'
        endif

	  Do i=1,nspec(0)
		Read(8,*) nspec(i), McNat(i), SuNat(i)
		Do j=1,nspec(i)
			n=n+1
			Natmc(n)=McNat(i)
			Natsu(n)=SuNat(i)
			natoms(n)=natmc(n)+natsu(n)
			nun(n)=natsu(n)/2
c
c		Check for dimensioning errors
c
		if(natoms(i).gt.maxatms) then
            write(10,40)
            write(10,*)'     requested ', natoms(i),' atoms'
            write(10,*)'    (requested ', nun(i),' repeat units)'
            stop 'readnew: # of atoms dimensioning error'
		endif

		End Do
	  Enddo
	  n = 0
	endif

   30 format(/,2x,'*** Number of chains dimensioning error ***')
   40 format(/,2x,'*** Number of atoms dimensioning error ***')

c
c     Read in cell size and coordinates of all chains
c
        do i = 1,3
          read(8,*) h(i,1), h(i,2), h(i,3)
        end do

	if(oldformat) then

	  Do ich=1,nchains

            Do iun=1,nun(ich)
              read(8,*) identmc(2*iun-1,ich),x0mc(2*iun-1,ich),
     &        y0mc(2*iun-1,ich),z0mc(2*iun-1,ich)
              read(8,*) identsu(2*iun-1,ich),x0su(2*iun-1,ich),
     &        y0su(2*iun-1,ich),z0su(2*iun-1,ich)
              read(8,*) identsu(2*iun  ,ich),x0su(2*iun  ,ich),
     &        y0su(2*iun  ,ich),z0su(2*iun  ,ich)
              read(8,*) identmc(2*iun  ,ich),x0mc(2*iun  ,ich),
     &        y0mc(2*iun  ,ich),z0mc(2*iun  ,ich)
            End Do
            read(8,*) identmc(natmc(ich),ich),x0mc(natmc(ich),ich),
     &                   y0mc(natmc(ich),ich),z0mc(natmc(ich),ich)
          End Do

        else

          Do ich=1,nchains
            Do iat=1,natmc(ich)
              read(8,*) identmc(iat,ich),x0mc(iat,ich),y0mc(iat,ich),
     &                     z0mc(iat,ich)
            End Do
          End Do

          Do ich=1,nchains
            Do iat=1,natsu(ich)
              read(8,*) identsu(iat,ich),x0su(iat,ich),y0su(iat,ich),
     &                     z0su(iat,ich)
            End Do
          End Do
	endif
c
c     total mass of box  (in units of mass H)
c
      totmass=0.0d0
      do  j=1,nchains
        do  i=1,natmc(j)
          totmass=totmass+umass(identmc(i,j))
	  end do
        do  i=1,natsu(j)
          totmass=totmass+umass(identsu(i,j))
	  end do
	end do
c
c     Set number of system degrees of freedom. Constraining the
c     bonds removes nprim-1 degrees of freedom from the polymer
c     and natompen-1 dof from the penetrant, otherwise
c     each penetrant adds 3 dof per unconstrained atom

	ndof = 0
      if(constraint) then
	  Do i = 1,nchains
	    ndof = ndof + natoms(i)+natoms(i)+1
	  End Do
      else
	  Do i = 1,nchains
	    ndof = ndof + natoms(i)+natoms(i)+natoms(i)
	  End Do
      endif

      if(nose) then
            ndof=ndof+3
      endif
c
c     And give atoms gaussian random velocities (x1,y1,z1)
c     Iseed is the initial seed for the random # generator. It can be 
c     any odd integer.
c
c
c     initialize variables for gaussdev and rranf
c
      iavail = 0
      told   = -1.0d0
      write(6,116)
  116 format(2x,'Assigning velocities to particles')
c
c     x1,y1,z1 in units of sqrt(1/mass(H))
c
	write(6,*) 'initial velocities? 1 for yes'
	read(8,*) initvel
	if(initvel.eq.1) read(8,*) iseed
	close(8)
      do 117 j=1,nchains
         do  i=1,natmc(j)
	   if(initvel.eq.1) then
            x1mc(i,j)=gaussdev(0,tstar/umass(identmc(i,j)),iseed)
            y1mc(i,j)=gaussdev(0,tstar/umass(identmc(i,j)),iseed)
            z1mc(i,j)=gaussdev(0,tstar/umass(identmc(i,j)),iseed)
c	Let it go without initial velocities
	    else
		x1mc(i,j)=0.0d0
		y1mc(i,j)=0.0d0
		z1mc(i,j)=0.0d0
	    endif
	  end do
         do  i=1,natsu(j)
	   if(initvel.eq.1) then
            x1su(i,j)=gaussdev(0,tstar/umass(identsu(i,j)),iseed)
            y1su(i,j)=gaussdev(0,tstar/umass(identsu(i,j)),iseed)
            z1su(i,j)=gaussdev(0,tstar/umass(identsu(i,j)),iseed)
	    else
		x1su(i,j)=0.0d0
		y1su(i,j)=0.0d0
		z1su(i,j)=0.0d0
	    endif
	  end do
 117  continue
c
c     total momentum in units of sqrt(mass(H))
c
      sumx=0.0d0
      sumy=0.0d0
      sumz=0.0d0
      do 120 j=1,nchains
         do  i=1,natmc(j)
            sumx=sumx+x1mc(i,j)*umass(identmc(i,j))
            sumy=sumy+y1mc(i,j)*umass(identmc(i,j))
            sumz=sumz+z1mc(i,j)*umass(identmc(i,j))
	  end do
         do  i=1,natsu(j)
            sumx=sumx+x1su(i,j)*umass(identsu(i,j))
            sumy=sumy+y1su(i,j)*umass(identsu(i,j))
            sumz=sumz+z1su(i,j)*umass(identsu(i,j))
	  end do
 120  continue

      sumx=sumx/totmass
      sumy=sumy/totmass
      sumz=sumz/totmass
c
c     Rescale velocities to get the desired temperature and zero net
c     translational momentum (sums).  Then convert to internal 
c     coordinates. Note: hi=inverse of h and vol=determinant
c
      CALL matinv(h,hi,vol)
      do 125 j=1,nchains
         do  i=1,natmc(j)
            x1mc(i,j)=x1mc(i,j)-sumx
            y1mc(i,j)=y1mc(i,j)-sumy
            z1mc(i,j)=z1mc(i,j)-sumz
	   end do
         do  i=1,natsu(j)
            x1su(i,j)=x1su(i,j)-sumx
            y1su(i,j)=y1su(i,j)-sumy
            z1su(i,j)=z1su(i,j)-sumz
	   end do
  125 continue
c
c     Calculate 2*kinetic energy in kcal/mol
c
      sumv2=0.0d0
      do 118 j=1,nchains
         do  i=1,natmc(j)
            sumv2= sumv2 + ( x1mc(i,j)*x1mc(i,j)
     >                     + y1mc(i,j)*y1mc(i,j)
     >                     + z1mc(i,j)*z1mc(i,j) ) 
     &                     * umass(identmc(i,j))
	  end do
         do  i=1,natsu(j)
            sumv2= sumv2 + ( x1su(i,j)*x1su(i,j)
     >                     + y1su(i,j)*y1su(i,j)
     >                     + z1su(i,j)*z1su(i,j) ) 
     &                     * umass(identsu(i,j))
	  end do
 118  continue
c
c     If using nose thermostat, only make temperature 0.05 of the 
c     desired temperature, then let heater bring it up slowly.
c     ndof = # of degrees of freedom
c     delta = time step in units of sqrt(mass(H)) ! formerly *sighh
c     heater = delta if at proper temperature
c     A recalculation of the temperature is shown where
c     ndof*kT/2 = mvv/2
c
	if (sumv2.eq.0.0d0) then
	  heater=1.0d0
	  go to 111
	endif
      heater=dsqrt(dfloat(ndof)*tstar/sumv2)*delta
  111 continue
c	next line used to be commented out - dont know why      
c	if(nose) heater=heater*0.05d0
      ttemp=0.0d0
      do 124 j=1,nchains
         do  i=1,natmc(j)
            xij1=x1mc(i,j)*heater 
            yij1=y1mc(i,j)*heater
            zij1=z1mc(i,j)*heater
c
c           Scale x1,y1,z1 by hi
c           remember x1=u'*delta ! 
c
            x1mc(i,j)=hi(1,1)*xij1+hi(1,2)*yij1+hi(1,3)*zij1
            y1mc(i,j)=hi(2,1)*xij1+hi(2,2)*yij1+hi(2,3)*zij1
            z1mc(i,j)=hi(3,1)*xij1+hi(3,2)*yij1+hi(3,3)*zij1
c
            xij1=(h(1,1)*x1mc(i,j)+h(1,2)*y1mc(i,j)+h(1,3)*z1mc(i,j))
     x	         /delta
            yij1=(h(2,1)*x1mc(i,j)+h(2,2)*y1mc(i,j)+h(2,3)*z1mc(i,j))
     x	         /delta
            zij1=(h(3,1)*x1mc(i,j)+h(3,2)*y1mc(i,j)+h(3,3)*z1mc(i,j))
     x           /delta
            ttemp=ttemp+(xij1*xij1+yij1*yij1+zij1*zij1)*
     x            umass(identmc(i,j))
	  end do
         do  i=1,natsu(j)
            xij1=x1su(i,j)*heater 
            yij1=y1su(i,j)*heater
            zij1=z1su(i,j)*heater
c
c           Scale x1,y1,z1 by hi
c           remember x1=u'*delta ! 
c
            x1su(i,j)=hi(1,1)*xij1+hi(1,2)*yij1+hi(1,3)*zij1
            y1su(i,j)=hi(2,1)*xij1+hi(2,2)*yij1+hi(2,3)*zij1
            z1su(i,j)=hi(3,1)*xij1+hi(3,2)*yij1+hi(3,3)*zij1
c
            xij1=(h(1,1)*x1su(i,j)+h(1,2)*y1su(i,j)+h(1,3)*z1su(i,j))
     x	         /delta
            yij1=(h(2,1)*x1su(i,j)+h(2,2)*y1su(i,j)+h(2,3)*z1su(i,j))
     x	         /delta
            zij1=(h(3,1)*x1su(i,j)+h(3,2)*y1su(i,j)+h(3,3)*z1su(i,j))
     x           /delta
            ttemp=ttemp+(xij1*xij1+yij1*yij1+zij1*zij1)*
     x            umass(identsu(i,j))
	 end do
  124 continue

      ttemp=ttemp/dfloat(ndof)*tmpscl
      write(6,200) heater,ndof,tstar,sumv2,delta*pstimscl,ttemp
  200 format(2x,'READING IN COORDS AND GENERATING V''S',/,
     x       2x,'heater = ',f10.6,/,2x,'ndof =',i5,/,2x,
     x       'tstar =',f10.4,/,2x,'2*KE =',f10.4,/,2x,
     x       'delta =',f10.6,' ps',/,2x,'Temperature = ',f10.6,' K')
c
c     Store initial shape tensor (used if cell changes volume)
c
      do 135 i=1,3
         do 135 j=1,3
            hinit(j,i)=h(j,i)
  135 continue
      return
      end
